<?php
/*
Plugin Name: PropertyPress IDX by BRANDco.
Description: MLS Search Integration
Author: BRANDco.
Author URI: https://brandco.com
Version: 2.5.1
License: GPLv2 or later
*/

require plugin_dir_path(__FILE__) . 'vendor/autoload.php';

define('BRANDCO_IDX_PLUGIN_VERSION', '2.5.1');
define('BRANDCO_IDX_PLUGIN_DIR_URL', plugin_dir_url(__FILE__));
define('BRANDCO_IDX_PLUGIN_PATH', plugin_dir_path(__FILE__));
define('BRANDCO_IDX_PLUGIN_REMOTE_INFO', 'https://hellobrandco.github.io/brandco-idx-plugin/plugin-info.json');

if ( !defined('PROPERTYPRESS_IDX_API') ) {
	define('PROPERTYPRESS_IDX_API', 'https://propertypress.com/wp-json/pp/v1');
}

if ( !defined('PROPERTYPRESS_IDX_AJAX') ) {
	define('PROPERTYPRESS_IDX_AJAX', '/wp-content/plugins/brandco-idx/propertypress-ajax.php');
}

if ( !defined('BRANDCO_IDX_ENV') ) {
	define('BRANDCO_IDX_ENV', 'live');
}

add_action('rest_api_init', array('BrandCo\Plugins\IDX\RestAPI', 'register'), 10, 1);

$bcoidx_ajax_calls = array(
	'find_user',
	'attemptAutocomplete',
	'toggle_email_notifications',
	'check_api_connection',
	'get_property_types',
	'search',
	'search_single',
	'contactForm',
	'login',
	'register',
	'save_settings',
	'create_hotsheet',
	'save_search',
	'autocomplete',
	'mls_metadata',
	'save_mls_metadata',
	'get_saved_searches',
	'favorite_listing',
	'unfavorite_listing',
	'unfavorite_saved_search',
	'get_my_favorites',
	'save_agent_metadata',
	'not_found_suggestions',
);

foreach ( $bcoidx_ajax_calls as $ajax_call ) {
	add_action('wp_ajax_bcoidx_ajax_' . $ajax_call, array('BrandCo\Plugins\IDX\Ajax', $ajax_call));
	add_action('wp_ajax_nopriv_bcoidx_ajax_' . $ajax_call, array('BrandCo\Plugins\IDX\Ajax', $ajax_call));
}

add_action('wp_enqueue_scripts', array('BrandCo\Plugins\IDX\Frontend', 'enqueue_js'), 5);
add_action('wp_enqueue_scripts', array('BrandCo\Plugins\IDX\Frontend', 'enqueue_css'));
add_action('admin_menu', array('BrandCo\Plugins\IDX\Settings', 'add_options_page'));
add_filter('generate_rewrite_rules', array('BrandCo\Plugins\IDX\Router', 'generate_rewrite_rules'), 10, 1);
add_filter('query_vars', array('BrandCo\Plugins\IDX\Router', 'query_vars'), 10, 1);
add_filter('template_include', array('BrandCo\Plugins\IDX\Router', 'template_include'), 5000, 1);
add_action('init', array('BrandCo\Plugins\IDX\IDXLeadRole', 'register'), 10, 1);
add_filter('show_admin_bar', array('BrandCo\Plugins\IDX\IDXLeadRole', 'hide_admin_bar'));
add_action('init', array('BrandCo\Plugins\IDX\Activity', 'register_post_type'), 10, 1);
add_action('init', array('BrandCo\Plugins\IDX\Hotsheets', 'register_post_type'), 10, 1);
add_action('init', array('BrandCo\Plugins\IDX\SavedSearches', 'register_post_type'), 10, 1);
add_action('init', array('BrandCo\Plugins\IDX\FavoriteListings', 'register_post_type'), 10, 1);
add_action('template_include', array('BrandCo\Plugins\IDX\Preload', 'fetch'), 5000, 1);
add_action('add_meta_boxes', array('BrandCo\Plugins\IDX\Hotsheets', 'metabox'), 10, 1);
add_action('add_meta_boxes', array('BrandCo\Plugins\IDX\SavedSearches', 'metabox'), 10, 1);
add_action('add_meta_boxes', array('BrandCo\Plugins\IDX\FavoriteListings', 'metabox'), 10, 1);
add_action('admin_head', array('BrandCo\Plugins\IDX\WPAdmin', 'shortcode_modal'), 10, 1);
add_action('edit_user_profile', array('BrandCo\Plugins\IDX\WPAdmin', 'profile_box'));

add_shortcode('idx_grid', array('BrandCo\Plugins\IDX\Shortcodes', 'listing_grid'));
add_shortcode('idx_slider', array('BrandCo\Plugins\IDX\Shortcodes', 'listing_slider'));
add_shortcode('idx_quick_search', array('BrandCo\Plugins\IDX\Shortcodes', 'search_widget'));
add_shortcode('idx_sold_listings', array('BrandCo\Plugins\IDX\Shortcodes', 'sold_listings_grid'));
add_shortcode('idx_featured_listings', array('BrandCo\Plugins\IDX\Shortcodes', 'featured_listings_grid'));
add_shortcode('idx_alert_signup', array('BrandCo\Plugins\IDX\Shortcodes', 'alert_signup_form'));

add_filter('pre_get_document_title', array('BrandCo\Plugins\IDX\SEO', 'set_title'), 999, 1);
add_filter('wp_head', array('BrandCo\Plugins\IDX\SEO', 'set_og_tags'), 1, 1);
add_filter('wp_head', array('BrandCo\Plugins\IDX\SEO', 'set_og_tags'), 999, 1);
add_filter('wpseo_canonical', array('BrandCo\Plugins\IDX\SEO', 'set_canonical'), 1);
add_filter('wpseo_opengraph_url', array('BrandCo\Plugins\IDX\SEO', 'set_canonical'), 1);

if ( !defined('WNT_FILE') ) {
	// wnt_list wnt_featured wnt_search wnt_grid
	add_shortcode('wnt_grid', array('BrandCo\Plugins\IDX\Shortcodes', 'wnt_grid'));
	add_shortcode('wnt_search', array('BrandCo\Plugins\IDX\Shortcodes', 'wnt_search'));
	add_shortcode('wnt_featured', array('BrandCo\Plugins\IDX\Shortcodes', 'wnt_featured'));
}

add_filter('body_class', function($classes) {

	if ( defined('BRANDCO_IDX_PAGE') ) {
		$classes[] = 'is-ppidx-page';
	}

	return $classes;
});


if ( BRANDCO_IDX_ENV == 'live' ) {
	add_filter('plugins_api', array('BrandCo\Plugins\IDX\Updater', 'info'), 20, 3);
	add_filter('site_transient_update_plugins', array('BrandCo\Plugins\IDX\Updater', 'update'));
	add_action('upgrader_process_complete', array('BrandCo\Plugins\IDX\Updater', 'after_update'), 10, 2);
	add_action('admin_head', array('BrandCo\Plugins\IDX\WPAdmin', 'css_hide_from_production'), 10, 1);
}


add_action('plugin_action_links_' . plugin_basename(__FILE__), 'propertypress_idx_plugin_links');
function propertypress_idx_plugin_links($links) {

	$links = array_merge( array(
		'<a href="' . esc_url(admin_url('/admin.php?page=propertypress_idx_settings_general')) . '">Settings</a>',
		'<a href="https://propertypress.com/download/" target="_blank">Help & Instructions</a>',
		// '<a href="' . esc_url('https://brandco.canny.io/changelog') . '" target="_blank">Changelog</a>'
	), $links );

	return $links;
}

add_action('bcoidx/actions/after_activity_logged', function($post_id) {
	$webhook = get_option('bcoidx_main_webhook');

	if ( $webhook ) {
		$activity = \BrandCo\Plugins\IDX\Activity::get_one($post_id);
		$request = wp_remote_post($webhook, array(
			'body' => $activity,
		));
	}
}, 10, 1);

add_action('nsl_register_new_user', array('BrandCo\Plugins\IDX\SocialLogin', 'after_register'));

add_filter('wpseo_sitemap_index', function ($sitemap_custom_items) {

	if ( get_option('bcoidx_xml_search') ) {
		$sitemap_custom_items .= '
			<sitemap>
				<loc>' . get_option('siteurl') . '/wp-json/idx/sitemap/listings</loc>
				<lastmod>' . date('Y-m-d\TH:i:s+00:00', time()) . '</lastmod>
			</sitemap>';
	}

	return $sitemap_custom_items;
});

add_filter('wpseo_canonical', 'propertypress_idx_override_yoast');
add_filter('wpseo_title', 'propertypress_idx_override_yoast');
add_filter('wpseo_metadesc', 'propertypress_idx_override_yoast');
add_filter('wpseo_opengraph_url', 'propertypress_idx_override_yoast');
add_filter('wpseo_opengraph_title', 'propertypress_idx_override_yoast');
add_filter('wpseo_opengraph_desc', 'propertypress_idx_override_yoast');
add_filter('wpseo_opengraph_image', 'propertypress_idx_override_yoast');

function propertypress_idx_override_yoast($string) {
	if ( defined('BRANDCO_IDX_PAGE') || defined('BRANDCO_IDX_SEARCH_PAGE') ) {
		return false;
	}

	return $string;
}
