<?php
namespace BrandCo\Plugins\IDX;

class Activity {

	public static function register_post_type() {
		$args = array(
			'public' => true,
			'supports' => array(
				'title',
				'editor',
				'thumbnail',
				'page-attributes',
				'revisions'
			),
			'label' => 'Activity',
			'has_archive' => true,
			'rewrite' => array(
				'slug' => 'bcoidx-activity'
			),
			'show_in_admin_bar' => false,
			'show_in_menu' => false,
			'show_in_nav_menus' => false,
			'show_ui' => false,
			'publicly_queryable' => false,
			'exclude_from_search' => true,
  		);

		register_post_type('bcoidx_activity', $args);
	}

	public static function log($wp_user_id, $subject, $message, $type, $person = array()) {

		$args = array(
			'post_title' => $subject,
			'post_content' => $message,
			'post_type' => 'bcoidx_activity',
			'post_status' => 'publish',
		);

		if ( $wp_user_id ) {
			$args['post_author'] = $wp_user_id;
			$user_info = get_userdata($wp_user_id);

			if ( empty($person['first']) ) {
				$person['first'] = $user_info->first_name;
			}

			if ( empty($person['last']) ) {
				$person['last'] = $user_info->last_name;
			}

			if ( empty($person['email']) ) {
				$person['email'] = $user_info->user_email;
			}

			if ( empty($person['phone']) ) {
				$person['phone'] = get_user_meta($wp_user_id, 'phone', true);
			}
		}

		$post_id = wp_insert_post($args);

		if ( is_wp_error($post_id) ) {
			return $post_id->get_error_message();
		}

		update_post_meta($post_id, 'activity_type', $type);
		update_post_meta($post_id, 'activity_for', $wp_user_id);

		if ( !empty($person['first']) ) {
			update_post_meta($post_id, 'first', $person['first']);
		}

		if ( !empty($person['last']) ) {
			update_post_meta($post_id, 'last', $person['last']);
		}

		if ( !empty($person['email']) ) {
			update_post_meta($post_id, 'email', $person['email']);
		}

		if ( !empty($person['phone']) ) {
			update_post_meta($post_id, 'phone', $person['phone']);
		}

		if ( !empty($message) ) {
			update_post_meta($post_id, 'note', $message);
		}

		if ( !empty($subject) ) {
			update_post_meta($post_id, 'subject', $subject);
		}

		do_action('bcoidx/actions/after_activity_logged', $post_id);

		return $post_id;
	}

	public static function get_my($user_id) {
		$activity = array();

		$query = new \WP_Query(array(
			'post_type' => 'bcoidx_activity',
			'posts_per_page' => 25,
			'author' => $user_id,
			'post_status' => 'publish'
		));

		if ( $query->have_posts() ) {
			while ( $query->have_posts() ) {
				$query->the_post();
				$result = get_post();

				$activity[] = array(
					'title' => $result->post_title,
					'wp_post_id' => $result->ID,
					'type' => get_post_meta($result->ID, 'activity_type', true)
				);
			}

			$query->reset_postdata();
		}

		return $activity;
	}

	public static function get_one($post_id) {
		$activity_type = get_post_meta($post_id, 'activity_type', true);
		$activity_for = get_post_meta($post_id, 'activity_for', true);
		$first = get_post_meta($post_id, 'first', true);
		$last = get_post_meta($post_id, 'last', true);
		$email = get_post_meta($post_id, 'email', true);
		$phone = get_post_meta($post_id, 'phone', true);
		$note = get_post_meta($post_id, 'note', true);
		$subject = get_post_meta($post_id, 'subject', true);
		return array(
			'activity_type' => $activity_type,
			'activity_for' => $activity_for,
			'first' => $first,
			'last' => $last,
			'email' => $email,
			'phone' => $phone,
			'note' => $note,
			'subject' => $subject,
		);
	}
}
