<?php
namespace BrandCo\Plugins\IDX;
use WP_Query;

class Ajax {

	public static function check_api_connection() {
		$request = new SearchListings([], true);
		// $response = $request->send_request_v1();

		wp_send_json_success(array(
			'request' => $request,
			'response' => $response,
		));
	}

	public static function get_property_types() {
		$mls_id = htmlspecialchars($_POST['mls_id']);

		$cached = get_transient("mls_property_types_{$mls_id}");
		if ( $cached ) {
			wp_send_json_success($cached);
			return;
		}

		$request = wp_remote_get('https://propertypress.com/wp-json/pp/v1/property-types?mls_id=' . $mls_id . '');
		$response = wp_remote_retrieve_body($request);
		$output = json_decode($response, true);

		if ( !empty($output['propertyType']) ) {
			set_transient("mls_property_types_{$mls_id}", $output, 72 * HOUR_IN_SECONDS);
		}

		wp_send_json_success($output);
	}

	public static function search($post) {

		$omni = ( !empty($post['omni']) ) ? true : false;
		$parameters = json_decode(html_entity_decode(stripslashes($post['parameters'])), true);
		$multi_params = array();
		$limit = ( !empty($parameters['limit']) ) ? $parameters['limit'] : 100;
		$mls_agent_list = $post['mls_agent_list'];
		$mls_office_list = $post['mls_office_list'];

		if ( !empty($post['remote_origin']) ) {
			global $remote_origin;
			$remote_origin = $post['remote_origin'];
		}

		// office only
		if ( !empty($parameters['officeOnly']) && $parameters['officeOnly'] == true && $mls_office_list ) {

			$all_listings = array();

			$total_count = 0;

			foreach ( $mls_office_list as $list_item ) {

				if ( $list_item['offices'] ) {
					$parameters['officeIds'] = $list_item['offices'];
					$parameters['mlsIds'] = array($list_item['mls']);
					$parameters['officeOnly'] = false;

					$request = new SearchListings($parameters, true, false, $limit);
					$response = $request->send_request_v2();

					$multi_params[] = $parameters;

					if ( !empty($response['response']['response']['listings_response']['listings']) ) {
						$all_listings = array_merge($all_listings, $response['response']['response']['listings_response']['listings']);
						$total_count += $response['response']['response']['count_response']['count'];
					}
				}
			}

			$response['response']['response']['count_response']['count'] = $total_count;
			$response['response']['response']['listings_response']['listings'] = $all_listings;

			$response = $request->remove_duplicate_listings($response);
			$response = $request->sort($response);

			$return = array(
				'multi_params' => $multi_params,
				'request' => $request,
				'response' => $response,
				'omni' => $omni
			);

			return $return;
		}

		// agent only
		if ( !empty($parameters['agentOnly']) && $parameters['agentOnly'] == true && $mls_agent_list ) {

			$all_listings = array();

			foreach ( $mls_agent_list as $list_item ) {

				if ( $list_item['agents'] ) {
					$parameters['agentIds'] = $list_item['agents'];
					$parameters['mlsIds'] = array($list_item['mls']);
					$parameters['agentOnly'] = false;
					$parameters['agentTypes'] = array('Listing Agent');

					$request = new SearchListings($parameters, true, false, $limit);
					$response = $request->send_request_v2();

					$multi_params[] = $parameters;

					if ( !empty($response['response']['response']['listings_response']['listings']) ) {
						$all_listings = array_merge($all_listings, $response['response']['response']['listings_response']['listings']);
					}
				}
			}

			$response['response']['response']['listings_response']['listings'] = $all_listings;

			$response = $request->remove_duplicate_listings($response);
			$response = $request->sort($response);

			$return = array(
				'multi_params' => $multi_params,
				'request' => $request,
				'response' => $response,
				'omni' => $omni
			);

			return $return;
		}

		$request = new SearchListings($parameters, $omni, false, $limit);
		$response = $request->send_request_v2();
		$response = $request->remove_duplicate_listings($response);
		$response = $request->sort($response);

		$return = array(
			'parameters' => $parameters,
			'request' => $request,
			'response' => $response,
			'omni' => $omni
		);

		return $return;

		// wp_send_json_success($return);
	}

	public static function search_single() {
		$mlsNum = $_POST['mlsNum'];

		$response = SingleListing::get($mlsNum);

		wp_send_json_success(array(
			'mlsNum' => $mlsNum,
			'response' => $response,
		));
	}

	public static function contactForm() {
		// check_ajax_referer('bcoidx', 'security');

		$parameters = json_decode(html_entity_decode(stripslashes($_POST['parameters'])), true);
		$to = ( get_option('bcoidx_main_email') ) ? get_option('bcoidx_main_email') : get_option('admin_email');
		$from_name = ( get_option('bcoidx_email_from_name') ) ? get_option('bcoidx_email_from_name') : get_option('blogname');
		// $subject = 'New IDX Lead | ' . $parameters['formTitle'] . ' (MLS ' . $parameters['mlsNum'] . ')';
		$subject = $_SERVER['SERVER_NAME'] . ', New IDX Lead';

		$message = Util::render_template('emails/contact-form-notification', $parameters);

		$email_2 = get_option('bcoidx_main_email_2');

		$headers = array(
			'From: ' . $from_name . ' <' . $to . '>',
			'Reply-To: ' . $parameters['first'] . ' ' . $parameters['last'] . ' <' . $parameters['email'] . '>',
			'Content-Type: text/html; charset=utf-8'
		);

		if ( get_option('bcoidx_email_type') === 'plain' ) {
			$message = Util::render_template('emails/contact-form-notification-plain', $parameters);
			$headers[2] = 'Content-Type: text/plain; charset=utf-8';
		}

		if ( $email_2 ) {
			$headers[] = 'Cc: ' . $email_2;
		}

		wp_mail($to, $subject, $message, $headers);

		$person = array(
			'first' => $parameters['first'],
			'last' => $parameters['last'],
			'email' => $parameters['email'],
			'phone' => $parameters['phone'],
		);

		if ( !empty($parameters['showing_date']) ) {
			$parameters['question'] .=  ' (Preferred date/time for showing: ' . $parameters['showing_date'] . ' @ ' . $parameters['showing_time'] . ')';
		}

		// if ( empty($parameters['question']) ) {
		// 	$parameters['question'] = 'No message';
		// }

		$activity_id = Activity::log(
			$parameters['wp_user_id'],
			$subject,
			$parameters['question'],
			'contact_form',
			$person
		);

		do_action('propertypressidx/activity/user_identified', array(
			'first_name' => $parameters['first'],
			'last_name' => $parameters['last'],
			'email_address' => $parameters['email'],
			'phone_number' => $parameters['phone'],
			'message' => $message,
		));

		wp_send_json_success(array(
			'parameters' => $parameters,
			'from_name' => $from_name,
			'to' => $to,
		));
	}

	public static function login() {
		// check_ajax_referer('bcoidx', 'security');

		$parameters = json_decode(base64_decode($_POST['parameters']), true);

		$creds = array();
		$creds['user_login'] = $parameters['user'];
		$creds['user_password'] = $parameters['pass'];

		$user = wp_signon($creds);

		if ( is_wp_error($user) ) {
			$response = array(
				'error' => "No user associated with your email address or the password was incorrect."
			);
		} else {

			$logged_in_user = array(
				'id' => $user->ID,
				'username' => $user->display_name,
				'role' => $user->roles[0],
				'firstname' => $user->first_name,
				'lastname' => $user->last_name,
				'emails_disabled' => ( get_user_meta($user->ID, 'bcoidx_emails_disabled', true) ) ? true : false,
			);

			// if ( is_super_admin() ) {
			// 	$logged_in_user['role'] = 'administrator';
			// }

			$response = array(
				'redirect' => Util::app_url('/'),
				'user' => $logged_in_user,
				'favorites' => FavoriteListings::get_my($user->ID),
			);
		}

		wp_send_json_success(array(
			'response' => $response,
		));
	}

	public static function register() {
		// check_ajax_referer('bcoidx', 'security');

		$parameters = json_decode(base64_decode($_POST['parameters']), true);

		$creds = array();
		$creds['user_login'] = sanitize_text_field($parameters['user']);
		$creds['user_email'] = sanitize_text_field($parameters['user']);
		$creds['first_name'] = sanitize_text_field($parameters['first']);
		$creds['last_name'] = sanitize_text_field($parameters['last']);
		$creds['display_name'] = sanitize_text_field($parameters['first']);

		if ( !empty($parameters['password']) ) {
			$creds['user_pass'] = $parameters['password'];
		} else {
			$creds['user_pass'] = wp_generate_password();
		}

		$creds['role'] = 'bcoidx_lead';

		$person = array(
			'first' => $parameters['first'],
			'last' => $parameters['last'],
			'email' => $parameters['email'],
			'phone' => $parameters['phone'],
			'note' => 'Registered'
		);

		$user = wp_insert_user($creds);

		if ( is_wp_error($user) ) {
			$response = array(
				'error' => $user->get_error_message()
			);
		} else {

			if ( !empty($parameters['phone']) ) {
				update_user_meta($user, 'ppidx_lead_phone', sanitize_text_field($parameters['phone']));
			}

			$_creds = array();
			$_creds['user_login'] = $creds['user_login'];
			$_creds['user_password'] = $creds['user_pass'];
			$_user = wp_signon($_creds);

			if ( empty($parameters['password']) ) {
				$user = get_user_by('id', $user);
				retrieve_password($user->user_login);
			}

			if ( is_wp_error($_user) ) {
				$response = array(
					'error' => $_user->get_error_message()
				);
			} else {

				$message = $creds['first_name'] . ' ' . $creds['last_name'];
				$message .= ' ' . $creds['user_email'];
				if ( !empty($parameters['phone']) ) {
					$message .= ' ' . $parameters['phone'];
				}
				$message .= ' registered.';

				$activity_id = Activity::log(
					$_user->ID,
					'New User Registration | ' . $creds['first_name'] . ' ' . $creds['last_name'] . ' ' . $creds['user_email'],
					$message,
					'user_registered',
					$person
				);

				$to = ( get_option('bcoidx_main_email') ) ? get_option('bcoidx_main_email') : get_option('admin_email');
				$subject = 'New IDX Lead Registration';
				$email_2 = get_option('bcoidx_main_email_2');
				$from_name = ( get_option('bcoidx_email_from_name') ) ? get_option('bcoidx_email_from_name') : get_option('blogname');

				$headers = array(
					'From: ' . $from_name . ' <' . $to . '>',
					'Content-Type: text/html; charset=UTF-8'
				);

				if ( $email_2 ) {
					$headers[] = 'Cc: ' . $email_2;
				}

				wp_mail($to, $subject, $message, $headers);

				do_action('propertypressidx/activity/user_identified', array(
					'first_name' => $creds['first_name'],
					'last_name' => $creds['last_name'],
					'email_address' => $creds['user_email'],
					'phone_number' => sanitize_text_field($parameters['phone']),
					'message' => 'User Registered',
				));

				$logged_in_user = array(
					'id' => $_user->ID,
					'username' => $_user->display_name,
					'role' => $_user->roles[0],
					'firstname' => $_user->first_name,
					'lastname' => $_user->last_name,
					'emails_disabled' => ( get_user_meta($_user->ID, 'bcoidx_emails_disabled', true) ) ? true : false,
				);

				if ( is_super_admin() ) {
					$logged_in_user['role'] = 'administrator';
				}

				$response = array(
					'redirect' => Util::app_url('/dashboard/?activity=' . $activity_id),
					'user' => $logged_in_user,
					'favorites' => FavoriteListings::get_my($user->ID),
				);
			}
		}

		wp_send_json_success(array(
			'response' => $response,
		));
	}

	public static function save_settings() {
		$settings = $_POST['settings'];
		delete_transient('ppidx_mls_metadata');

		$mls_metadata = MLSMetadata::get();

		foreach ( $settings as $setting ) {

			if ( strpos($setting['name'], 'bcoidx_agent_id_') !== false ) {
				foreach ( $mls_metadata as $_mls ) {
					$_mls = reset($_mls);
					if ( $setting['name'] == 'bcoidx_agent_id_' . $_mls['mlsId'] ) {
						$value = sanitize_text_field($setting['value']);
						$value = str_replace(' ', '', $value);
						update_option($setting['name'], $value);
					}
				}
			}

			if ( strpos($setting['name'], 'bcoidx_office_id_') !== false ) {
				foreach ( $mls_metadata as $_mls ) {
					$_mls = reset($_mls);
					if ( $setting['name'] == 'bcoidx_office_id_' . $_mls['mlsId'] ) {
						$value = sanitize_text_field($setting['value']);
						$value = str_replace(' ', '', $value);
						update_option($setting['name'], $value);
					}
				}
			}

			if ( 'bcoidx_api_key' == $setting['name'] ) {
				update_option($setting['name'], base64_encode(sanitize_text_field($setting['value'])));
			}

			if ( 'bcoidx_app_url' == $setting['name'] ) {
				update_option($setting['name'], $setting['value']);
			}

			if ( 'bcoidx_main_email' == $setting['name'] ) {
				update_option($setting['name'], sanitize_email($setting['value']));
			}

			if ( 'bcoidx_main_webhook' == $setting['name'] ) {
				update_option($setting['name'], sanitize_text_field($setting['value']));
			}

			if ( 'bcoidx_main_email_2' == $setting['name'] ) {
				update_option($setting['name'], sanitize_email($setting['value']));
			}

			if ( 'bcoidx_none_found_text' == $setting['name'] ) {
				update_option(
					$setting['name'],
					wp_kses(
						$setting['value'],
						array(
							'a' => array(
								'href' => array(),
								'title' => array()
							),
							'br' => array(),
							'em' => array(),
							'strong' => array(),
						)
					)
				);
			}

			if ( 'bcoidx_email_footer' == $setting['name'] ) {
				update_option(
					$setting['name'],
					wp_kses(
						$setting['value'],
						array(
							'a' => array(
								'href' => array(),
								'title' => array()
							),
							'br' => array(),
							'em' => array(),
							'strong' => array(),
						)
					)
				);
			}

			// if ( 'bcoidx_disable_rounded' == $setting['name'] ) {
			// 	var_dump($setting);
			// 	if ( $setting['value'] == true ) {
			// 		update_option($setting['name'], true);
			// 	} else {
			// 		update_option($setting['name'], false);
			// 	}
			// } else {
			// 	update_option($setting['name'], false);
			// }

			// update_option($setting['name'], false);

			if (
				'bcoidx_email_from_name' == $setting['name'] ||
				'bcoidx_google_maps_api' == $setting['name'] ||
				'bcoidx_recaptcha_site_key' == $setting['name'] ||
				'bcoidx_cta_disclaimer' == $setting['name'] ||
				'bcoidx_email_type' == $setting['name'] ||
				'bcoidx_app_test_url' == $setting['name'] ||
				'bcoidx_forced_reg_type' == $setting['name'] ||
				'bcoidx_forced_reg_listing_count' == $setting['name'] ||
				'bcoidx_show_duplicate_listings' == $setting['name'] ||
				'bcoidx_hide_top_courtesy' == $setting['name'] ||
				'bcoidx_mls' == $setting['name'] ||
				'bcoidx_agent_id' == $setting['name'] ||
				'bcoidx_initial_search' == $setting['name'] ||
				'bcoidx_fallback_search' == $setting['name'] ||
				'bcoidx_xml_search' == $setting['name'] ||
				'bcoidx_fallback_listing_photo' == $setting['name'] ||
				'bcoidx_logo_branding' == $setting['name'] ||
				'bcoidx_logo_branding_id' == $setting['name'] ||
				'bcoidx_office_id' == $setting['name'] ||
				'bcoidx_layout_options' == $setting['name'] ||
				'bcoidx_default_mls_status' == $setting['name'] ||
				'bcoidx_primary_color' == $setting['name'] ) {
				update_option($setting['name'], sanitize_text_field($setting['value']));
			}
		}

		wp_send_json_success(array(
			'settings' => $settings,
		));
	}

	public static function create_hotsheet() {

		$title = (!empty($_POST['title'])) ? sanitize_text_field($_POST['title']) : 'Hotsheet ' . date('F j, Y, g:i a');

		$post_id = wp_insert_post(array(
			'post_title' => $title,
			'post_status' => 'publish',
			'post_type' => 'bcoidx_hotsheets',
		));

		if ( !empty($_POST['parameters']) ) {
			$parameters = $_POST['parameters'];
			update_post_meta($post_id, 'bcoidx_hotsheet_search_parameters', $parameters);
		}

		if ( !empty($_POST['neighborhood']) ) {
			$neighborhood = $_POST['neighborhood'];
			update_post_meta($post_id, 'bcoidx_hotsheet_neighborhood', $neighborhood);
		}

		wp_send_json_success(array(
			'parameters' => $parameters,
		));
	}

	public static function attemptAutocomplete()
	{
		$search_text = htmlspecialchars($_POST['search_text']);

		if ( !empty($_POST['remote_origin']) ) {
			global $remote_origin;
			$remote_origin = $_POST['remote_origin'];
		}

		$request = new AutocompleteRequest(array(
			'statuses' => 'active,pending,activeUnderContract',
			'fields' => 'city,state',
			'suggests' => 'cityState',
			'prefix' => $search_text,
		));

		$response = $request->send_request_v2();
		$suggestions = $response['response']['response']['response'];

		wp_send_json_success(array(
			'success' => true,
			'suggestions' => $suggestions,
		));
	}

	public static function autocomplete() {
		// check_ajax_referer('bcoidx', 'security');

		$search = htmlspecialchars($_POST['search']);

		if ( !empty($_POST['remote_origin']) ) {
			global $remote_origin;
			$remote_origin = $_POST['remote_origin'];
		}

		$args = array(
			// 'mlses' => trim(get_option('bcoidx_mls')),
			'statuses' => 'active,pending,activeUnderContract,sold',
			'fields' => 'address,districtCityState',
			'suggests' => 'postalCode,cityState,districtCityState,countyState,buildingName,fullAddress,ln,elementarySchool,middleSchool,highSchool,mlsAreaMajor,schoolDistrict',
			'prefix' => $search,
		);

		if ( !empty($_POST['mlses']) ) {
			$args['mlses'] = $_POST['mlses'];
		}

		$request = new AutocompleteRequest($args);

		$response = $request->send_request_v2();

		$suggestions = $response['response']['response']['response'];
		$suggestions_array = array();
		$category = '';
		foreach ( $suggestions as $suggestion ) {

			if ( $suggestion['suggestType'] == 'districtCityState' ) {
				$suggestion['text'] = $suggestion['fieldValues']['districtCityState'];
			}

			if ( in_array( $suggestion['suggestType'], array('districtCityState', 'cityState', 'countyState') ) ) {
				if ( strpos($suggestion['text'], ',') == false ) {
					continue;
				}
			}

			if ( $category !== $suggestion['suggestType'] ) {
				$category = $suggestion['suggestType'];
				$suggestions_array[] = $category;
			}

			$suggestions_array[] = $suggestion;
		}

		wp_send_json_success(array(
			'search' => $search,
			'request' => $request,
			'suggestions_array' => $suggestions_array,
		));
	}

	public static function save_agent_metadata() {
		$json = $_POST['json'];
		$agent_id = $_POST['agent_id'];

		update_option('bcoidx_agent_' . $agent_id . '_metadata', base64_encode($json));

		wp_send_json_success(array(
			'success' => true,
		));
	}

	public static function mls_metadata() {
		$mls = sanitize_text_field($_POST['mls']);
		$request = new RequestMLSMetadata($mls);
		$response = $request->send_request();

		wp_send_json_success(array(
			'mls' => $mls,
			'request' => $request,
			'response' => $response,
		));
	}

	public static function save_mls_metadata() {
		$json = $_POST['json'];
		$mls = $_POST['mls'];

		update_option('bcoidx_mls_' . $mls . '_metadata', base64_encode($json));

		wp_send_json_success(array(
			'success' => true,
		));
	}

	public static function save_search() {
		// check_ajax_referer('bcoidx', 'security');

		$parameters = $_POST['parameters'];
		$wp_user_id = $_POST['user_id'];
		$alert_title = ( !empty($_POST['title']) ) ? sanitize_text_field($_POST['title']) : 'Saved Search ' . date('F j, Y, g:i a');
		$wp_user = get_user_by('id', $wp_user_id);
		$search_link = Util::app_url() . "?q={$parameters}";

		$email_params = [
			'formTitle' => "Saved Search {$alert_title}",
			'first' => $wp_user->first_name,
			'last' => $wp_user->last_name,
			'email' => $wp_user->user_email,
			'phone' => get_user_meta($wp_user_id, 'phone', true),
			'question' => "Saved Search: {$alert_title} <a href='{$search_link}'>View Search</a>",
			'link' => '',
			'streetAddress' => '',
			'mlsNum' => '',
		];

		$initial_params = Util::decode_params($parameters);

		$body = array();
		$body['wp_user_id'] = $wp_user_id;
		$body['searchName'] = $alert_title;

		if ( !empty($initial_params['priceMax']) ) {
			$body['priceMax'] = $initial_params['priceMax'];
		}

		if ( !empty($initial_params['priceMin']) ) {
			$body['priceMin'] = $initial_params['priceMin'];
		}

		if ( !empty($initial_params['propertyTypes']) ) {
			$body['propertyTypes'] = $initial_params['propertyTypes'];
		}

		$body['polygon'] = $initial_params['polygon'];

		$args = array(
			'method' => 'POST',
			'body' => json_encode($body),
			'headers' => array(
				'Authorization' => 'Bearer ' . base64_encode(Util::get_origin_domain()),
				'Content-Type' => 'application/json'
			),
			'timeout' => 15,
		);

		$endpoint = PROPERTYPRESS_IDX_API . '/register-alert';

		$request = wp_remote_request($endpoint, $args);

		if ( is_wp_error($request) || wp_remote_retrieve_response_code($request) !== 200 ) {
			return $request;
		}

		$response = wp_remote_retrieve_body($request);
		$response = json_decode($response, true);

		if ( !empty($response['alertId']) ) {

			(new Mail)->send('IDX Lead | Saved Search', $email_params);

			$post_id = wp_insert_post(array(
				'post_title' => $alert_title,
				'post_status' => 'publish',
				'post_type' => 'bcoidx_savedsearch',
				'post_author' => (int) $wp_user_id,
			));

			update_post_meta($post_id, 'bcoidx_saved_search_search_parameters', $parameters);
			update_post_meta($post_id, 'bcoidx_saved_search_brivityalertid', $response['alertId']);

			Activity::log(
				$wp_user_id,
				$alert_title,
				$wp_user_id,
				'saved_search'
			);

			wp_send_json_success(array(
				'saved_search' => $response['alertId'],
			));
		} else {
			wp_send_json_error(array(
				'sent_body' => $body,
				'brivity_response' => $response,
			));
		}
	}

	public static function favorite_listing() {
		// check_ajax_referer('bcoidx', 'security');

		$mlsnum = $_POST['mlsnum'];
		// $mlsid = $_POST['mlsid'];
		$wp_user_id = $_POST['user_id'];
		$alert_title = $_POST['title'];
		$wp_user = get_user_by('id', $wp_user_id);

		$body = array(
			'wp_user_id' => $wp_user_id,
			'searchName' => $alert_title,
			'mlsNums' => array($mlsnum),
		);

		$args = array(
			'method' => 'POST',
			'body' => json_encode($body),
			'headers' => array(
				'Authorization' => 'Bearer ' . base64_encode(Util::get_origin_domain()),
				'Content-Type' => 'application/json'
			),
			'timeout' => 15,
		);

		$endpoint = PROPERTYPRESS_IDX_API . '/register-alert';

		$request = wp_remote_request($endpoint, $args);

		if ( is_wp_error($request) || wp_remote_retrieve_response_code($request) !== 200 ) {
			return $request;
		}

		$response = wp_remote_retrieve_body($request);
		$response = json_decode($response, true);

		if ( !empty($response['alertId']) ) {
			$post_id = wp_insert_post(array(
				'post_title' => $alert_title,
				'post_status' => 'publish',
				'post_type' => 'bcoidx_favlistings',
				'post_author' => (int) $wp_user_id,
			));

			update_post_meta($post_id, 'bcoidx_favorite_listing_mlsnum', $mlsnum);
			update_post_meta($post_id, 'bcoidx_favorite_listing_mlsid', $mlsid);
			update_post_meta($post_id, 'bcoidx_favorite_listing_brivityalertid', $response['alertId']);

			(new Mail)->send('IDX Lead | Favorited Listing', [
				'formTitle' => "Favorited Listing {$alert_title}",
				'first' => $wp_user->first_name,
				'last' => $wp_user->last_name,
				'email' => $wp_user->user_email,
				'phone' => $wp_user->user_phone,
				'question' => "Favorited Listing: {$alert_title}",
				'link' => get_option('siteurl') . '/' . get_option('bcoidx_app_url') . '/listing/listing--' . $mlsnum,
				'streetAddress' => $alert_title,
				'mlsNum' => $mlsnum,
			]);

			Activity::log(
				$wp_user_id,
				'Favorited Listing: ' . $alert_title,
				$mlsnum,
				'favorited_listing'
			);

			wp_send_json_success(array(
				'saved_listing' => $post_id,
				'new_favorites' => FavoriteListings::get_my($wp_user_id),
			));
		} else {
			wp_send_json_error($response);
		}
	}

	public static function unfavorite_listing() {
		// check_ajax_referer('bcoidx', 'security');

		$favorite_post_id = $_POST['favorite_post_id'];
		$br_alert_id = get_post_meta($favorite_post_id, 'bcoidx_favorite_listing_brivityalertid', true);
		$wp_user_id = $_POST['user_id'];

		$args = array(
			'br_alert_id' => $br_alert_id,
		);

		$args = array(
			'method' => 'POST',
			'body' => json_encode($args),
			'headers' => array(
				'Authorization' => 'Bearer ' . base64_encode(Util::get_origin_domain()),
				'Content-Type' => 'application/json'
			),
			'timeout' => 15,
		);

		$endpoint = PROPERTYPRESS_IDX_API . '/delete-alert';
		$request = wp_remote_request($endpoint, $args);
		$response = wp_remote_retrieve_body($request);
		wp_trash_post($favorite_post_id);

		wp_send_json_success(array(
			'deleted' => array(
				$favorite_post_id,
				$br_alert_id,
			),
			'new_favorites' => FavoriteListings::get_my($wp_user_id),
		));
	}

	public static function unfavorite_saved_search() {
		// check_ajax_referer('bcoidx', 'security');

		$favorite_post_id = $_POST['favorite_post_id'];
		$br_alert_id = get_post_meta($favorite_post_id, 'bcoidx_saved_search_brivityalertid', true);
		$wp_user_id = $_POST['user_id'];

		$args = array(
			'br_alert_id' => $br_alert_id,
		);

		$args = array(
			'method' => 'POST',
			'body' => json_encode($args),
			'headers' => array(
				'Authorization' => 'Bearer ' . base64_encode(Util::get_origin_domain()),
				'Content-Type' => 'application/json'
			),
			'timeout' => 15,
		);

		$endpoint = PROPERTYPRESS_IDX_API . '/delete-alert';
		$request = wp_remote_request($endpoint, $args);
		$response = wp_remote_retrieve_body($request);
		wp_trash_post($favorite_post_id);

		wp_send_json_success(array(
			'deleted' => array(
				$favorite_post_id,
				$br_alert_id,
			),
			'new_list' => SavedSearches::get_my($wp_user_id),
		));
	}

	public static function get_saved_searches() {
		$user_id = $_POST['user_id'];
		$query = new WP_Query(array(
			'post_type' => 'bcoidx_savedsearch',
			'posts_per_page' => -1,
			'author' => $user_id,
		));

		$save_searches = array();

		if ( $query->have_posts() ) {
			while ( $query->have_posts() ) {
				$query->the_post();
				$result = get_post();
				$save_searches[] = array(
					'id' => $result->ID,
					'parameters' => get_post_meta($result->ID, 'bcoidx_saved_search_search_parameters', true),
					'alertid' => get_post_meta($result->ID, 'bcoidx_saved_search_brivityalertid', true),
					'created' => get_the_date('F j, Y, g:i a', $result->ID),
					'title' => get_the_title(),
				);
			}

			$query->reset_postdata();
		}

		wp_send_json_success(array(
			'save_searches' => $save_searches,
		));
	}

	public static function get_my_favorites() {
		$user_id = $_POST['user_id'];

		wp_send_json_success(array(
			'my_favorites' => FavoriteListings::get_my($user_id),
		));
	}

	public static function toggle_email_notifications() {

		if ( !empty($_POST['user_email']) ) {
			$user = get_user_by_email($_POST['user_email']);

			if ( !$user ) {
				wp_send_json_error(array(
					'error' => 'No user found'
				));
			} else {
				update_user_meta($user->ID, 'bcoidx_emails_disabled', true);
				wp_send_json_success(array(
					'unsubbed' => true,
					'user' => $user->user_email,
				));
			}
		} else {
			$user = wp_get_current_user();
		}

		$emails_disabled = get_user_meta($user->ID, 'bcoidx_emails_disabled', true);

		if ( $emails_disabled ) {
			$change_to = false;
		} else {
			$change_to = true;
		}

		update_user_meta($user->ID, 'bcoidx_emails_disabled', $change_to);

		wp_send_json_success(array(
			'bcoidx_emails_disabled_now' => $change_to,
			'user' => $user->user_email,
		));
	}

	public static function not_found_suggestions() {
		$query = array();
		if ( !empty($_POST['address']) ) {

			$request = new AutocompleteRequest(array(
				'statuses' => 'active,pending,activeUnderContract',
				'fields' => 'address,ln',
				'suggests' => 'districtCityState,postalCode,cityState,ln,elementarySchool,middleSchool,highSchool,mlsAreaMajor,fullAddress,schoolDistrict,countyState,buildingName',
				'prefix' => $_POST['address'],
			));

			$response = $request->send_request_v2();

			$suggestions = $response['response']['response']['response'];

			if ( $suggestions ) {
				$suggestion = $suggestions[0];

				if ( 'fullAddress' == $suggestion['suggestType'] ) {
					$query['mlsNums'] = array($suggestion['fieldValues']['ln']);
				}
			}
		}

		wp_send_json_success(array(
			'query' => $query,
		));
	}

	public static function find_user() {
		check_ajax_referer('bcoidx', 'security');

		$email = htmlspecialchars($_POST['email']);

		if ( !is_email($email) ) {
			wp_send_json_error([
				'message' => 'Not a valid email'
			]);
		}

		$user = get_user_by('email', $email);

		wp_send_json_success([
			'user' => $user
		]);

	}
}
