<?php
namespace BrandCo\Plugins\IDX;

class Frontend {

	public static function enqueue_js() {

		if ( !defined('BRANDCO_IDX_PAGE') ) {
			return;
		}

		global $bcoidx_mls_metadata;

		if ( !$bcoidx_mls_metadata ) {
			$bcoidx_mls_metadata = MLSMetadata::get();
		}

		if ( !$bcoidx_mls_metadata ) {
			return;
		}

		$mls_agent_list = Util::mls_agent_list($bcoidx_mls_metadata);
		$mls_office_list = Util::mls_office_list($bcoidx_mls_metadata);

		$site_extension = get_option('siteurl');
		$site_extension = str_replace('https://', '', $site_extension);
		$site_extension = str_replace('http://', '', $site_extension);
		$site_extension = str_replace($_SERVER['HTTP_HOST'], '', $site_extension);

		$mlses = array();
		if ( is_array($bcoidx_mls_metadata) ) {
			foreach ( $bcoidx_mls_metadata as $mls_data ) {
				if ( !empty($mls_data) ) {
					$mls_data = $mls_data[array_keys($mls_data)[0]]; // sorry
					$mlses[$mls_data['mlsId']] = $mls_data;
				}
			}
		}

		if ( !empty($_GET['action']) && $_GET['action'] == 'elementor' ) {
			return;
		}

		wp_register_script('brandco-idx-shortcodes-js', BRANDCO_IDX_PLUGIN_DIR_URL . 'public/js/brandco-idx-shortcodes.js', array('jquery'), BRANDCO_IDX_PLUGIN_VERSION, true);

		wp_localize_script('brandco-idx-shortcodes-js', 'bcoidx', array(
			'fast_ajax' => PROPERTYPRESS_IDX_AJAX,
			'ajax' => admin_url('admin-ajax.php'),
			'security' => wp_create_nonce('bcoidx'),
			'home' => $site_extension . '/' . get_option('bcoidx_app_url'),
			'none_found_text' => ( get_option('bcoidx_none_found_text') ) ? get_option('bcoidx_none_found_text') : 'No listings found.',
			'fallback_search' => ( get_option('bcoidx_fallback_search') ) ? get_option('bcoidx_fallback_search') : '',
			'google_maps_api' => ( get_option('bcoidx_google_maps_api') ) ? get_option('bcoidx_google_maps_api') : 'AIzaSyBj0R77qlkoyl56fTKK-5_1gsvvIu4bEEk',
			'layout_options' => get_option('bcoidx_layout_options'),
			'default_mls_status' => get_option('bcoidx_default_mls_status'),
			'cta_disclaimer' => get_option('bcoidx_cta_disclaimer'),
			'mls_agent_list' => $mls_agent_list,
			'mls_office_list' => $mls_office_list,
			'mlses' => $mlses,
		));

		if ( !defined('BRANDCO_IDX_PAGE') ) {
			return;
		}

		if ( BRANDCO_IDX_PAGE === 'shortcode' ) {
			return;
		}

		wp_deregister_script('swiper');
		wp_dequeue_script('swiper');
		wp_enqueue_script('brandco-idx-js', BRANDCO_IDX_PLUGIN_DIR_URL . 'public/js/brandco-idx.js', array('jquery'), BRANDCO_IDX_PLUGIN_VERSION, true);

		if ( get_option('bcoidx_recaptcha_site_key') ) {
			wp_enqueue_script('brandco-idx-recaptcha',
				'https://www.google.com/recaptcha/api.js?onload=vueRecaptchaApiLoaded&render=explicit',
				array('brandco-idx-js'), null, true
			);
		}

		$user = wp_get_current_user();

		$localized_data = array(
 			'siteurl' => get_option('siteurl'),
			'fast_ajax' => PROPERTYPRESS_IDX_AJAX,
			'ajax' => admin_url('admin-ajax.php'),
			'security' => wp_create_nonce('bcoidx'),
			'home' => $site_extension . '/' . get_option('bcoidx_app_url'),
			'logout' => wp_logout_url(get_option('siteurl') . '/' . get_option('bcoidx_app_url')),
			'google_maps_api' => ( get_option('bcoidx_google_maps_api') ) ? get_option('bcoidx_google_maps_api') : 'AIzaSyBj0R77qlkoyl56fTKK-5_1gsvvIu4bEEk',
			'recaptcha_site_key' => get_option('bcoidx_recaptcha_site_key'),
			'layout_options' => get_option('bcoidx_layout_options'),
			'default_mls_status' => get_option('bcoidx_default_mls_status'),
			'cta_disclaimer' => get_option('bcoidx_cta_disclaimer'),
			'none_found_text' => ( get_option('bcoidx_none_found_text') ) ? get_option('bcoidx_none_found_text') : 'No listings found.',
			'primary_color' => get_option('bcoidx_primary_color'),
			'notification_email' => ( get_option('bcoidx_main_email') ) ? get_option('bcoidx_main_email') : get_option('admin_email'),
			'idx_show_closed_option' => ( !empty($_GET['idx-extras']) ) ? 1 : false,
			'forced_reg_type' => get_option('bcoidx_forced_reg_type'),
			'forced_reg_count' => get_option('bcoidx_forced_reg_listing_count'),
			'hide_top_courtesy' => ( get_option('bcoidx_hide_top_courtesy') == 'yes' ) ? 'yes' : 'no',
			'user' => array(
				'id' => '',
				'username' => '',
				'role' => '',
				'firstname' => '',
				'lastname' => '',
				'emails_disabled' => ''
 			),
 			'favorites' => array(),
		);

		$fallback_listing_photo = get_option('bcoidx_fallback_listing_photo');

		if ( $fallback_listing_photo ) {
			$localized_data['fallback_listing_photo'] = $fallback_listing_photo;
		}

		$mls_metadata = $bcoidx_mls_metadata;

		$localized_data['mlses'] = array();
		$localized_data['mls_list'] = array();

		foreach ( $mls_metadata as $mls_data ) {
			if ( !empty($mls_data) ) {
				$mls_data = $mls_data[array_keys($mls_data)[0]]; // sorry
				$localized_data['mls_list'][] = $mls_data['mlsId'];
				$localized_data['mlses'][$mls_data['mlsId']] = $mls_data;

				$searchFilters = array();

				foreach ( $mls_data['searchFields'] as $fields ) {
					if ( $fields['searchCategory'] ) {
						$label = $fields['category'];

						if ( $label == 'Home Style' ) {
							$label = 'Property Style';
						}

						$searchFilters[$fields['searchCategory']] = array(
							'key' => $fields['searchCategory'],
							'label' => $label,
						);
					}
				}

				foreach ( $mls_data['searchFields'] as $fields ) {
					if ( $fields['searchCategory'] ) {
						$searchFilters[$fields['searchCategory']]['options'][] = array(
							'term' => $fields['searchValue'],
							'excludeTerm' => false,
							'matchAny' => true,
						);
					}
				}

				$localized_data['mlses'][$mls_data['mlsId']]['extraFilters'] = $searchFilters;
			}
		}

		$mls_agent_list = Util::mls_agent_list($mls_metadata);
		$mls_office_list = Util::mls_office_list($mls_metadata);

		$localized_data['mls_agent_list'] = $mls_agent_list;
		$localized_data['mls_office_list'] = $mls_office_list;
		$localized_data['agent_ids'] = array();
		$localized_data['agents'] = array();

		$office_ids = array();
		if ( $mls_office_list ) {
			foreach ( $mls_office_list as $mls ) {
				if ( !empty($mls['offices']) ) {
					foreach ( $mls['offices'] as $office_id ) {
						$office_ids[] = $office_id;
					}
				}
			}
		}

		$localized_data['office_ids'] = $office_ids;

		if ( $mls_agent_list ) {
			foreach ( $mls_agent_list as $mls ) {
				if ( !empty($mls['agents']) ) {
					$agent_ids = $mls['agents'];

					foreach ( $agent_ids as $agent_id ) {
						$localized_data['agent_ids'][] = $agent_id;

						$agent_metadata = get_option('bcoidx_agent_' . $agent_id . '_metadata');

						if ( $agent_metadata ) {
							$agent_metadata = stripslashes(html_entity_decode(base64_decode($agent_metadata)));
							$agent_metadata = json_decode($agent_metadata, true);
							$localized_data['agents'][$agent_id] = $agent_metadata;
						}
					}
				}
			}
		}

		if ( is_user_logged_in() ) {

			$emails_disabled = get_user_meta($user->ID, 'bcoidx_emails_disabled', true);

			$localized_data['user'] = array(
				'id' => $user->ID,
				'username' => $user->display_name,
				'role' => $user->roles[0],
				'firstname' => $user->first_name,
				'lastname' => $user->last_name,
				'emails_disabled' => ( $emails_disabled ) ? true : false,
			);

			if ( is_super_admin() ) {
				$localized_data['user']['role'] = 'administrator';
			}

			$localized_data['favorites'] = FavoriteListings::get_my($user->ID);
		}

		// Nextend Integration
		$localized_data['nextend_is_active'] = 'no';
		$localized_data['nextend_providers'] = '';

		if ( class_exists('NextendSocialLogin') ) {
			$providers = array_keys(\NextendSocialLogin::$enabledProviders);
			$localized_data['nextend_providers'] = $providers;
			$localized_data['nextend_is_active'] = 'yes';
		}

		wp_localize_script('brandco-idx-js', 'bcoidx', $localized_data);
	}

	public static function enqueue_css() {
		$primary_color = ( get_option('bcoidx_primary_color') ) ? get_option('bcoidx_primary_color') : '#D50000';
		$primary_color_dark = Util::adjustColor($primary_color, -50);
		$custom_css = "
			:root {
				--propertypressIDXprimaryColor: " . $primary_color . ";
				--propertypressIDXprimaryColorDark: " . $primary_color_dark . ";
			}
		";

		wp_register_style('brandco-idx-shortcodes-css', BRANDCO_IDX_PLUGIN_DIR_URL . 'public/css/brandco-idx-shortcodes.css', array(), BRANDCO_IDX_PLUGIN_VERSION, 'all');

		if ( !defined('BRANDCO_IDX_PAGE') ) {
			return;
		}

		wp_enqueue_style('brandco-idx-css', BRANDCO_IDX_PLUGIN_DIR_URL . 'public/css/brandco-idx.css', array(), BRANDCO_IDX_PLUGIN_VERSION, 'all');
		wp_add_inline_style('brandco-idx-css', $custom_css);
	}
}
