<?php
namespace BrandCo\Plugins\IDX;

class Hotsheets {

	public static function register_post_type() {
		$public = false;

		if ( strpos($_SERVER['SERVER_NAME'], '.local') !== false ) {
			$public = true;
		}

		$args = array(
			'public' => true,
			'supports' => array(
				'revisions',
				'title',
				// 'editor',
				'author',
			),
			'label' => 'Hotsheets',
			'has_archive' => false,
			'rewrite' => array(
				'slug' => get_option('bcoidx_app_url') . '-page'
			),
			'show_in_admin_bar' => $public,
			'show_in_menu' => $public,
			'show_in_nav_menus' => $public,
			'show_ui' => true,
			'publicly_queryable' => true,
			'exclude_from_search' => true,
  		);

		register_post_type('bcoidx_hotsheets', $args);
	}

	public static function metabox() {
		add_meta_box(
			'bcoidx_hotsheets_metabox',
			'Hotsheet Details',
			__NAMESPACE__ . '\Hotsheets::metabox_content',
			'bcoidx_hotsheets'
		);
	}

	public static function get_public_search_parameters($post_id) {
		$encoded_params = get_post_meta($post_id, 'bcoidx_hotsheet_search_parameters', true);

		if ( !$encoded_params ) {
			// echo 'Invalid';
		}

		$parameters = json_decode(base64_decode($encoded_params), true);

		$public_keys = [
			'postalCodes',
			'propertyTypes',
			'priceMin',
			'priceMax',
			'bedsMin',
			'bathsMin',
			'yearMin',
			'lotSizeMin',
			'sqFtMin',
			'sqFtMax',
			'statuses',
			'countyStates',
			'streetAddress',
			'cityStates',
			'neighborhoodCityStates',
		];

		$public_parameters = array();

		if ( $parameters ) {
	  		foreach ($parameters as $key => $value) {
				if ( in_array($key, $public_keys) ) {

					$public_parameters[$key] = json_encode($value);
				}
			}
		}

		return $public_parameters;
	}

	public static function metabox_content($post) {
		$encoded_params = get_post_meta($post->ID, 'bcoidx_hotsheet_search_parameters', true);

		if ( !$encoded_params ) {
			echo 'Invalid';
		}

		$parameters = json_decode(base64_decode($encoded_params), true);

		$public_keys = [
			'propertyTypes',
			'priceMin',
			'priceMax',
			'bedsMin',
			'bathsMin',
		];

		?>
			<style>
				/*#publish,
				.wp-heading-inline,
				.page-title-action {
					display: none !important;
				}*/
			</style>
			<table class="form-table">
				<?php foreach ($parameters as $key => $value): ?>
					<?php if ( in_array($key, $public_keys) ) : ?>
						<tr>
							<th><?php echo $key; ?></th>
							<td>
								<?php
									if ( is_array($value) ) {
										echo implode(', ', $value);
									} else {
										echo $value;
									}
								?>
							</td>
						</tr>
					<?php endif; ?>
				<?php endforeach; ?>
				<tr>
					<th>Search String</th>
					<td><textarea readonly="readonly" class="regular-text" style="height: 300px; font-size: 12px;"><?php echo $encoded_params; ?></textarea></td>
				</tr>
			</table>
		<?php
	}
}
