<?php
namespace BrandCo\Plugins\IDX;

class ListingAlert {

	public static function save($args) {
		$parameters = $args['parameters'];
		$wp_user_id = $args['user_id'];
		$alert_title = ( !empty($args['title']) ) ? sanitize_text_field($args['title']) : 'Saved Search ' . date('F j, Y, g:i a');

		$initial_params = Util::decode_params($parameters);

		$body = array();
		$body['wp_user_id'] = $wp_user_id;
		$body['searchName'] = $alert_title;

		if ( !empty($initial_params['priceMax']) ) {
			$body['priceMax'] = $initial_params['priceMax'];
		}

		if ( !empty($initial_params['priceMin']) ) {
			$body['priceMin'] = $initial_params['priceMin'];
		}

		if ( !empty($initial_params['bedsMin']) ) {
			$body['bedsMin'] = $initial_params['bedsMin'];
		}

		if ( !empty($initial_params['bathsMin']) ) {
			$body['bathsMin'] = $initial_params['bathsMin'];
		}

		if ( !empty($initial_params['sqFtMin']) ) {
			$body['sqFtMin'] = $initial_params['sqFtMin'];
		}

		if ( !empty($initial_params['sqFtMax']) ) {
			$body['sqFtMax'] = $initial_params['sqFtMax'];
		}

		if ( !empty($initial_params['lotSizeMin']) ) {
			$body['lotSizeMin'] = $initial_params['lotSizeMin'];
		}

		// Residential, Condo/Townhome
		if ( !empty($initial_params['propertyTypes']) ) {
			$body['propertyTypes'] = $initial_params['propertyTypes'];
		}

		if ( !empty($initial_params['cityStates']) ) {
			$body['cityStates'] = $initial_params['cityStates'];
		}

		if ( !empty($initial_params['postalCodes']) ) {
			$body['postalCodes'] = $initial_params['postalCodes'];
		}

		if ( !empty($initial_params['neighborhoodCityStates']) ) {
			$body['neighborhoodCityStates'] = $initial_params['neighborhoodCityStates'];
		}

		$args = array(
			'method' => 'POST',
			'body' => json_encode($body),
			'headers' => array(
				'Authorization' => 'Bearer ' . base64_encode(Util::get_origin_domain()),
				'Content-Type' => 'application/json'
			),
			'timeout' => 15,
		);

		$endpoint = PROPERTYPRESS_IDX_API . '/register-alert';

		$request = wp_remote_request($endpoint, $args);

		if ( is_wp_error($request) || wp_remote_retrieve_response_code($request) !== 200 ) {
			return $request;
		}

		$response = wp_remote_retrieve_body($request);
		$response = json_decode($response, true);

		if ( !empty($response['alertId']) ) {

			$post_id = wp_insert_post(array(
				'post_title' => $alert_title,
				'post_status' => 'publish',
				'post_type' => 'bcoidx_savedsearch',
				'post_author' => (int) $wp_user_id,
			));

			update_post_meta($post_id, 'bcoidx_saved_search_search_parameters', $parameters);
			update_post_meta($post_id, 'bcoidx_saved_search_brivityalertid', $response['alertId']);

			Activity::log(
				$wp_user_id,
				$alert_title,
				'Search Saved #' . $post_id,
				'saved_search'
			);

			return array(
				'saved_search' => $response['alertId'],
				'alert_pid' => $post_id,
			);
		} else {
			return array(
				'sent_body' => $body,
				'brivity_response' => $response,
			);
		}
	}

	public static function remove($favorite_post_id) {
		$br_alert_id = get_post_meta($favorite_post_id, 'bcoidx_saved_search_brivityalertid', true);

		$args = array(
			'method' => 'POST',
			'body' => json_encode([
				'br_alert_id' => $br_alert_id,
			]),
			'headers' => array(
				'Authorization' => 'Bearer ' . base64_encode(Util::get_origin_domain()),
				'Content-Type' => 'application/json'
			),
			'timeout' => 15,
		);

		$endpoint = PROPERTYPRESS_IDX_API . '/delete-alert';
		$request = wp_remote_request($endpoint, $args);
		$response = wp_remote_retrieve_body($request);
		wp_trash_post($favorite_post_id);

		return [
			'deleted_pid' => $favorite_post_id,
			'deleted_alert_id' => $br_alert_id,
		];
	}

	public static function delete($br_alert_id) {
		$args = array(
			'method' => 'POST',
			'body' => json_encode([
				'br_alert_id' => $br_alert_id,
			]),
			'headers' => array(
				'Authorization' => 'Bearer ' . base64_encode(Util::get_origin_domain()),
				'Content-Type' => 'application/json'
			),
			'timeout' => 15,
		);

		$endpoint = PROPERTYPRESS_IDX_API . '/delete-alert';
		$request = wp_remote_request($endpoint, $args);
		$response = wp_remote_retrieve_body($request);

		return [
			'deleted_alert_id' => $br_alert_id,
		];
	}
}
