<?php
namespace BrandCo\Plugins\IDX;
use WP_REST_Server, WP_REST_Request;

class RestAPI {

	public static function register() {

		register_rest_route('idx/v1', '/test', array(
			'methods'  => WP_REST_Server::READABLE,
			'callback' => __CLASS__ . '::test',
			'permission_callback' => '__return_true',
		));

		register_rest_route('idx/v1', '/testing', array(
			'methods'  => WP_REST_Server::READABLE,
			'callback' => __CLASS__ . '::testing',
			'permission_callback' => '__return_true',
		));

		register_rest_route('idx', '/import-alert', array(
			'methods'  => WP_REST_Server::CREATABLE,
			'callback' => __CLASS__ . '::import_alert',
			'permission_callback' => '__return_true',
		));

		register_rest_route('idx', '/remove-alert', array(
			'methods'  => WP_REST_Server::CREATABLE,
			'callback' => __CLASS__ . '::remove_alert',
			'permission_callback' => '__return_true',
		));

		register_rest_route('idx/v1', '/receive-alert-message', array(
			'methods'  => WP_REST_Server::CREATABLE,
			'callback' => __CLASS__ . '::receive_alert_message',
			'permission_callback' => '__return_true',
		));

		register_rest_route('idx/v1', '/dummy-response/(?P<type>\S+)', array(
			'methods'  => WP_REST_Server::CREATABLE,
			'callback' => __CLASS__ . '::dummy_response',
			'permission_callback' => '__return_true',
		));

		register_rest_route('idx/v1', '/dummy-response/(?P<type>\S+)', array(
			'methods'  => WP_REST_Server::READABLE,
			'callback' => __CLASS__ . '::dummy_response',
			'permission_callback' => '__return_true',
		));

		register_rest_route('idx/v1', '/fake-response', array(
			'methods'  => WP_REST_Server::CREATABLE,
			'callback' => __CLASS__ . '::fake_response',
			'permission_callback' => '__return_true',
		));

		register_rest_route('idx/v1', '/fake-autocomplete-response', array(
			'methods'  => WP_REST_Server::CREATABLE,
			'callback' => __CLASS__ . '::fake_autocomplete_response',
			'permission_callback' => '__return_true',
		));

		register_rest_route('idx/v1', '/fake-mls-data-response', array(
			'methods'  => WP_REST_Server::CREATABLE,
			'callback' => __CLASS__ . '::fake_mls_data_response',
			'permission_callback' => '__return_true',
		));

		register_rest_route('idx/v1', '/fake-count-response', array(
			'methods'  => WP_REST_Server::CREATABLE,
			'callback' => __CLASS__ . '::fake_count_response',
			'permission_callback' => '__return_true',
		));

		register_rest_route('idx/v1', '/fake-listings-search-response', array(
			'methods'  => WP_REST_Server::CREATABLE,
			'callback' => __CLASS__ . '::fake_listings_search_response',
			'permission_callback' => '__return_true',
		));

		register_rest_route('idx/v1', '/shortcodes/agent-slider', array(
			'methods'  => WP_REST_Server::READABLE,
			'callback' => __CLASS__ . '::shortcode_agent_slider',
			'permission_callback' => '__return_true',
		));

		register_rest_route('idx/v1', '/shortcodes/make', array(
			'methods'  => WP_REST_Server::READABLE,
			'callback' => __CLASS__ . '::shortcode_make',
			'permission_callback' => '__return_true',
		));

		register_rest_route('idx/sitemap/', '/listings', array(
			'methods'  => WP_REST_Server::READABLE,
			'callback' => __CLASS__ . '::sitemap',
			'permission_callback' => '__return_true',
		));
	}

	public static function sitemap(WP_REST_Request $request) {
		$body = $request->get_json_params();
		$bcoidx_xml_search = get_option('bcoidx_xml_search');
		$searchParams = Util::decode_params($bcoidx_xml_search);

		foreach ( $searchParams as $key => $param ) {
			if ( isset($param['fields']) && empty($param['fields']) ) {
				unset($searchParams[$key]);
			}
		}

		unset($searchParams['showListingsWithoutPhotos']);
		unset($searchParams['page']);
		unset($searchParams['zoom']);
		unset($searchParams['center']);
		unset($searchParams['limit']);
		unset($searchParams['mapIsHidden']);
		unset($searchParams['mapHasShape']);
		unset($searchParams['mlsIds']);

		$searchParams['queryOptions']['select'] = [
			'streetAddress',
			'mlsNum',
			'updateDate',
		];

		$listingSearch = new SearchListings($searchParams, false, true, 500);
		$response = $listingSearch->send_request_v2();
		$listings = $response['response']['response']['listings_response']['listings'];

		ob_start();
			echo '<?xml version="1.0" encoding="UTF-8"?>';
			?>
			<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9" total="<?php echo count($listings); ?>">
				<?php foreach ( $listings as $listing ) : ?>
					<?php
						$listingLink = get_option('siteurl') . '/' .
							get_option('bcoidx_app_url') .
							'/listing/' . sanitize_title($listing['streetAddress']) . '--' . $listing['mlsNum'];
					?>
					<url>
						<loc><?php echo $listingLink; ?></loc>
						<lastmod><?php echo $listing['updateDate']; ?></lastmod>
					</url>
				<?php endforeach; ?>
			</urlset>
		<?php $sitemap = ob_get_contents();
		ob_end_clean();

		if ( ! $request->get_param('testing') ) {
			header('Content-type: text/xml');
			echo $sitemap;
			exit;
		}

		return rest_ensure_response([
			'searched' => $response['response']['brivity_request'],
			'listings' => $listings,
			'sitemap' => $sitemap,
		]);
	}

	public static function shortcode_make(WP_REST_Request $request) {
		ob_start();
			echo do_shortcode($request->get_param('shortcode'));
		$html = ob_get_contents();
		ob_end_clean();

		$bcoidx_mls_metadata = MLSMetadata::get();
		$mls_agent_list = Util::mls_agent_list($bcoidx_mls_metadata);
		$mls_office_list = Util::mls_office_list($bcoidx_mls_metadata);
		$site_extension = get_option('siteurl');
		// $site_extension = str_replace('https://', '', $site_extension);
		// $site_extension = str_replace('http://', '', $site_extension);
		// $site_extension = str_replace($_SERVER['HTTP_HOST'], '', $site_extension);

		$mlses = array();
		if ( is_array($bcoidx_mls_metadata) ) {
			foreach ( $bcoidx_mls_metadata as $mls_data ) {
				$mls_data = $mls_data[array_keys($mls_data)[0]]; // sorry
				$mlses[$mls_data['mlsId']] = $mls_data;
			}
		}

		return rest_ensure_response([
			'html' => $html,
			'window' => array(
				'fast_ajax' => PROPERTYPRESS_IDX_AJAX,
				'ajax' => admin_url('admin-ajax.php'),
				'security' => wp_create_nonce('bcoidx'),
				'home' => $site_extension . '/' . get_option('bcoidx_app_url'),
				'none_found_text' => ( get_option('bcoidx_none_found_text') ) ? get_option('bcoidx_none_found_text') : 'No listings found.',
				'fallback_search' => ( get_option('bcoidx_fallback_search') ) ? get_option('bcoidx_fallback_search') : '',
				'layout_options' => get_option('bcoidx_layout_options'),
				'primary_color' => ( get_option('bcoidx_primary_color') ) ? get_option('bcoidx_primary_color') : '#D50000',
				'mls_agent_list' => $mls_agent_list,
				'mls_office_list' => $mls_office_list,
				'mlses' => $mlses,
			)
		]);
	}

	public static function shortcode_agent_slider(WP_REST_Request $request) {

		ob_start();
			echo do_shortcode('[idx_quick_search layout="simple"]');
		$quick_search_html = ob_get_contents();
		ob_end_clean();

		ob_start();
			echo do_shortcode('[idx_slider for="agent"]');
		$agent_slider_html = ob_get_contents();
		ob_end_clean();

		$bcoidx_mls_metadata = MLSMetadata::get();

		$mls_agent_list = Util::mls_agent_list($bcoidx_mls_metadata);
		$mls_office_list = Util::mls_office_list($bcoidx_mls_metadata);

		$site_extension = get_option('siteurl');
		$site_extension = str_replace('https://', '', $site_extension);
		$site_extension = str_replace('http://', '', $site_extension);
		$site_extension = str_replace($_SERVER['HTTP_HOST'], '', $site_extension);

		$mlses = array();
		if ( is_array($bcoidx_mls_metadata) ) {
			foreach ( $bcoidx_mls_metadata as $mls_data ) {
				$mls_data = $mls_data[array_keys($mls_data)[0]]; // sorry
				$mlses[$mls_data['mlsId']] = $mls_data;
			}
		}

		return rest_ensure_response(array(
			'quick_search_html' => $quick_search_html,
			'agent_slider_html' => $agent_slider_html,
			'window' => array(
				'fast_ajax' => PROPERTYPRESS_IDX_AJAX,
				'ajax' => admin_url('admin-ajax.php'),
				'security' => wp_create_nonce('bcoidx'),
				'home' => $site_extension . '/' . get_option('bcoidx_app_url'),
				'none_found_text' => ( get_option('bcoidx_none_found_text') ) ? get_option('bcoidx_none_found_text') : 'No listings found.',
				'fallback_search' => ( get_option('bcoidx_fallback_search') ) ? get_option('bcoidx_fallback_search') : '',
				'layout_options' => get_option('bcoidx_layout_options'),
				'mls_agent_list' => $mls_agent_list,
				'mls_office_list' => $mls_office_list,
				'mlses' => $mlses,
			)
		));
	}

	public static function dummy_response(WP_REST_Request $request) {
		$type = $request->get_param('type');
		$json = file_get_contents(BRANDCO_IDX_PLUGIN_PATH . 'dummy-responses/' . $type . '.json');
		return rest_ensure_response(json_decode($json, true));
	}

	public static function fake_response(WP_REST_Request $request) {
		$json = file_get_contents(BRANDCO_IDX_PLUGIN_PATH . 'fake-response.json');
		return rest_ensure_response(json_decode($json, true));
	}

	public static function fake_autocomplete_response(WP_REST_Request $request) {
		$json = file_get_contents(BRANDCO_IDX_PLUGIN_PATH . 'fake-autocomplete-response.json');
		return rest_ensure_response(json_decode($json, true));
	}

	public static function fake_mls_data_response(WP_REST_Request $request) {
		$json = file_get_contents(BRANDCO_IDX_PLUGIN_PATH . 'fake-mls-data-response.json');
		return rest_ensure_response(json_decode($json, true));
	}

	public static function fake_count_response(WP_REST_Request $request) {
		$json = json_encode(array(
			'count' => 112358,
		));
		return rest_ensure_response(json_decode($json, true));
	}

	public static function import_alert(WP_REST_Request $request) {
		$email = $request->get_param('email');
		$first = $request->get_param('first');
		$last = $request->get_param('last');
		$idx_search = $request->get_param('idx_search');
		$search_title = $request->get_param('search_title');

		$wp_user = get_user_by('email', $email);

		if ( !$wp_user ) {
			$wp_user_id = wp_insert_user([
				'user_email' => $email,
				'first_name' => $first,
				'last_name' => $last,
				'user_login' => $email,
				'role' => 'bcoidx_lead',
			]);
		} else {
			$wp_user_id = $wp_user->ID;
		}

		$alert = ListingAlert::save([
			'parameters' => base64_encode(json_encode($idx_search)),
			'user_id' => $wp_user_id,
			'title' => $search_title,
		]);

		return rest_ensure_response($alert);
	}

	public static function remove_alert(WP_REST_Request $request) {
		$alert_pid = $request->get_param('alert_pid');

		if ( get_post_type($alert_pid) == 'bcoidx_savedsearch' ) {
			$alert = ListingAlert::remove($alert_pid);
		}

		return rest_ensure_response([
			'alert' => $alert,
		]);
	}

	public static function testing(WP_REST_Request $request) {
		$request = new SearchListings([

		], false, true, 20);

		// $request = new AutocompleteRequest(array(
		// 	'statuses' => 'active,pending,activeUnderContract',
		// 	'fields' => 'address,ln',
		// 	'suggests' => 'districtCityState,postalCode,cityState,ln,elementarySchool,middleSchool,highSchool,mlsAreaMajor,fullAddress,schoolDistrict,countyState,buildingName',
		// 	'prefix' => 'park',
		// ));

		$response = $request->send_request_v2();
		$response = $request->remove_duplicate_listings($response);
		$response = $request->sort($response);

		return rest_ensure_response($response);

		// $email_alert = new AlertEmail;

		// $alert_id = 'Mt1i4X8BU2Fq-GJ67_cn';

		// $email_sent = $email_alert->process_email('max@brandco.com', array(
		// 	'listing' => array(
		// 		"alertType" => "price_change",
		// 		"mlsNum" => "1002498",
		// 		"blossorId" => "39-1002498",
		// 		"photoDomain" => "https://photos.brivity.com",
		// 		"area" => "GTW",
		// 		"city" => "Georgetown",
		// 		"streetAddress" => "1816 Flying Horseshoe BND",
		// 		"standardStatus" => "Active",
		// 		"state" => "TX",
		// 		"postalCode" => "78628",
		// 		"price" => 468440
		// 	),
		// ), $alert_id);

		$args = Util::request_args('GET');
		// $args['headers']['Authorization'] = '';
		$endpoint = PROPERTYPRESS_IDX_API . '/mls-metadata/';
		$request = wp_remote_request($endpoint, $args);
		$response = wp_remote_retrieve_body($request);
		$response = json_decode($response, true);

		return rest_ensure_response(array(
			'args' => $args,
			'response' => $response,
			'error' => empty($response['error']),
		));
	}

	public static function test(WP_REST_Request $request) {
		$mls_metadata = MLSMetadata::get();
		$origin = Util::get_origin_domain();
		$requestargs = Util::request_args('GET');

		$dir = plugin_dir_path(__FILE__);
		$ajax_file = str_replace('/lib/', '/propertypress-ajax.php', $dir);

		if ( !empty($request['fixajax']) ) {
			chmod($ajax_file, 0644);
		}

		$url = PROPERTYPRESS_IDX_API . '/test/';
		$request = wp_remote_request($url, Util::request_args('GET'));
		$body = wp_remote_retrieve_body($request);

		return rest_ensure_response(array(
			'mls_metadata' => $mls_metadata,
			'version' => BRANDCO_IDX_PLUGIN_VERSION,
			'origin' => $origin,
			'endpoint' => PROPERTYPRESS_IDX_API,
			'requestargs' => $requestargs,
			'url' => $url,
			'request' => $request,
			'body' => json_decode($body, true),
			'ajax_file_check' => array(
				'file' => $ajax_file,
				'exists' => is_file($ajax_file),
				'current_perms' => substr(sprintf('%o', fileperms($ajax_file)), -4),
			)
		));
	}

	public static function fake_listings_search_response(WP_REST_Request $request) {
		$json = file_get_contents(BRANDCO_IDX_PLUGIN_PATH . 'fake-listings-search-response.json');
		return rest_ensure_response(json_decode($json, true));
	}

	public static function receive_alert_message(WP_REST_Request $request) {
		$body = $request->get_json_params();

		$wp_user_id = $body['wp_user_id'];
		$br_alert_id = $body['br_alert_id'];
		$br_message = $body['br_message'];

		$wp_user = get_user_by('id', $wp_user_id);

		if ( !$wp_user ) {
			ListingAlert::delete($br_alert_id);
			return rest_ensure_response(array(
				'error' => 'No user found',
			));
		}

		$bcoidx_emails_disabled = get_user_meta($wp_user_id, 'bcoidx_emails_disabled', true);

		if ( $bcoidx_emails_disabled ) {
			ListingAlert::delete($br_alert_id);
			return rest_ensure_response(array('error' => 'Email notifications disabled'));
		}

		$post_found_for_alert = false;
		$stored_alerts = Util::get_my_alerts($wp_user->ID);

		foreach ( $stored_alerts as $stored_alert ) {
			if ( $stored_alert['br_id'] == $br_alert_id ) {
				$post_found_for_alert = true;
			}
		}

		if ( $post_found_for_alert ) {
			$email_alert = new AlertEmail;
			$email_sent = $email_alert->process_email(
				$wp_user->user_email,
				$br_message,
				$br_alert_id
			);

			if ( $email_sent ) {
				Activity::log(
					$wp_user_id,
					'Alert Email Sent (' . $br_alert_id . ')',
					strtoupper($br_message['listing']['alertType']) . ' at ' . $br_message['listing']['streetAddress'],
					'listing_alert_email_sent'
				);

				$return = array(
					'success' => 'Alert sent to ' . $wp_user->user_email
				);
			} else {
				$return = array(
					'error' => 'WP email error'
				);
			}
		} else {
			ListingAlert::delete($br_alert_id);
			$return = array(
				'error' => 'Alert Deleted'
			);
		}

		return rest_ensure_response($return);
	}
}
