<?php
namespace BrandCo\Plugins\IDX;
use WP_Query;

class SavedSearches {

	public static function register_post_type() {
		$args = array(
			'public' => true,
			'supports' => array(
				'revisions'
			),
			'label' => 'Saved Searches',
			'has_archive' => true,
			'rewrite' => array(
				'slug' => 'bcoidx-saved-searches'
			),
			'show_in_admin_bar' => true,
			'show_in_menu' => true,
			'show_in_nav_menus' => true,
			'show_ui' => true,
			'publicly_queryable' => true,
			'exclude_from_search' => true,
  		);

		register_post_type('bcoidx_savedsearch', $args);
	}

	public static function metabox() {
		add_meta_box(
			'bcoidx_savedsearch_metabox',
			'Saved Search Details',
			__NAMESPACE__ . '\SavedSearches::metabox_content',
			'bcoidx_savedsearch'
		);
	}

	public static function metabox_content($post) {
		$encoded_params = get_post_meta($post->ID, 'bcoidx_saved_search_search_parameters', true);

		if ( !$encoded_params ) {
			echo 'Invalid';
		}

		$parameters = json_decode(base64_decode($encoded_params), true);

		$public_keys = [
			'propertyTypes',
			'priceMin',
			'priceMax',
			'bedsMin',
			'bathsMin',
		];

		?>
			<style>
				#publish,
				.wp-heading-inline,
				.page-title-action {
					display: none !important;
				}
			</style>
			<table class="form-table">
				<?php foreach ($parameters as $key => $value): ?>
					<?php if ( in_array($key, $public_keys) ) : ?>
						<tr>
							<th><?php echo $key; ?></th>
							<td>
								<?php
									if ( is_array($value) ) {
										echo implode(', ', $value);
									} else {
										echo $value;
									}
								?>
							</td>
						</tr>
					<?php endif; ?>
				<?php endforeach; ?>
				<tr>
					<th>Search String</th>
					<td><input type="text" value="<?php echo $encoded_params; ?>" class="regular-text" readonly="readonly"></td>
				</tr>
			</table>
		<?php
	}

	public static function get_my($user_id) {
		$saved = array();

		$query = new WP_Query(array(
			'post_type' => 'bcoidx_savedsearch',
			'posts_per_page' => -1,
			'author' => $user_id,
			'post_status' => 'publish'
		));

		if ( $query->have_posts() ) {
			while ( $query->have_posts() ) {
				$query->the_post();
				$result = get_post();

				$parameters = get_post_meta($result->ID, 'bcoidx_saved_search_search_parameters', true);

				$saved[] = array(
					'parameters' => Util::decode_params($parameters),
					'param_string' => $parameters,
					'title' => $result->post_title,
					'wp_post_id' => $result->ID,
				);
			}

			$query->reset_postdata();
		}

		return $saved;
	}
}
