<?php
namespace BrandCo\Plugins\IDX;

class SearchListings {
	public $body = array();
	public $original_body;

	public function __construct($body = array(), $omni = false, $all_fields = false, $limit = 100) {

		$this->original_body = $body;

		$body['queryOptions']['limit'] = $limit;

		if ( !$all_fields ) {
			$body['queryOptions']['select'] = array(
				'streetAddress',
				'city',
				'state',
				'postalCode',
				'squareFeet',
				'bedrooms',
				'bathsTotalDecimal',
				'price',
				'mlsNum',
				'lat',
				'lng',
				'photosAvailable',
				'photos',
				'originatingSystemName',
				'blueroofMlsId',
				'listAgentId',
				'listAgentName',
				'listOfficeId',
				'listOfficeName',
				'soldDate',
				'soldPrice',
				'standardStatus',
				'mlsStatus',
				'standardPropertyType',
				'openhouses',
				'listDate',
				'hideAddress',
			);
		}

		if ( !empty($body['agentOnly']) ) {

			$omni = true; // Sometimes listings outside current map view

			if ( true == $body['agentOnly'] ) {
				// $agent_ids =  get_option('bcoidx_agent_id');
				// $agent_ids = explode(',', $agent_ids);

				$bcoidx_mls_metadata = MLSMetadata::get();
				$mls_agent_list = Util::mls_agent_list($bcoidx_mls_metadata);
				$agent_ids = array_merge(...array_map(function($agent) {
					return $agent['agents'];
				}, $mls_agent_list));

				$body['agentIds'] = $agent_ids;
			}
		}

		if ( !empty($body['officeOnly']) ) {
			$omni = true; // Sometimes listings outside current map view

			unset($body['agentIds']); // prioritize office checkbox

			if ( true == $body['officeOnly'] ) {
				// $office_ids =  get_option('bcoidx_office_id');

				$bcoidx_mls_metadata = MLSMetadata::get();
				$mls_office_list = Util::mls_office_list($bcoidx_mls_metadata);
				$office_ids = array_merge(...array_map(function($office) {
					return $office['offices'];
				}, $mls_office_list));

				$body['officeIds'] = $office_ids;
			}
		}

		if ( !empty($body['priceMin']) && 0 == $body['priceMin'] ) {
			unset($body['priceMin']);
		}

		if ( !empty($body['priceMax']) && 100000000 == $body['priceMax'] ) {
			unset($body['priceMax']);
		}

		if ( empty($body['propertyTypes']) ) {
			unset($body['propertyTypes']);
		}

		if ( empty($body['agentIds']) ) {
			unset($body['agentIds']);
		}

		if ( empty($body['officeIds']) ) {
			unset($body['officeIds']);
		}

		if ( empty($body['statuses']) ) {
			$body['statuses'] = array(1, 3);
		}

		$body['requirePhotos'] = true;
		if ( !empty($body['showListingsWithoutPhotos']) ) {
			$body['requirePhotos'] = false;
		}

		if ( empty($body['yearMin']) ) {
			unset($body['yearMin']);
		}

		if ( !empty($body['yearMin']) && $body['yearMin'] == 0 ) {
			unset($body['yearMin']);
		}

		if ( empty($body['lotSizeMin']) ) {
			unset($body['lotSizeMin']);
		}

		if ( !empty($body['lotSizeMin']) && $body['lotSizeMin'] == 0 ) {
			unset($body['lotSizeMin']);
		}

		// $body['mlsIds'] = Util::get_mls_array();
		// $body['statuses'] = array(1);

		if ( !empty($body['mlsNums']) ) {
			// unset($body['statuses']);
			$body['statuses'] = array(1,2,3,4);
		}

		foreach ( $body as $key => $param ) {
			if ( is_array($param) && empty($param) ) {
				unset($body[$key]);
			}

			if ( is_array($param) && array_key_exists('fields', $param) && empty($param['fields']) ) {
				unset($body[$key]);
			}
		}

		// $body['queryOptions']['offset'] = 0;

		// if ( !empty($body['page']) ) {
		// 	$body['queryOptions']['offset'] = $limit * $body['page'];
		// }

		if ( !empty($body['mapIsHidden']) && $body['mapIsHidden'] == true ) {
			unset($body['polygon']);
		}

		if ($omni) {
			unset($body['polygon']);
		}

		if ( !empty($body['mapHasShape']) ) {
			$body['polygon'] = $body['mapHasShape'];
		}

		unset($body['showListingsWithoutPhotos']);
		unset($body['page']);
		unset($body['zoom']);
		unset($body['center']);
		unset($body['limit']);
		unset($body['mapIsHidden']);
		unset($body['mapHasShape']);

		// if ( !empty($body['agentOnly']) ) {

		// 	if ( true == $body['agentOnly'] ) {
		// 		$body = array();

		// 		$mls_metadata = MLSMetadata::get();

		// 		$agent_ids = array();

		// 		// foreach ( $mls_metadata as $_mls ) {
		// 		// 	$_mls = reset($_mls);

		// 		// 	$agent_id = get_option('bcoidx_agent_id_' . $_mls['mlsId']);

		// 		// 	if ( $agent_id ) {
		// 		// 		$agent_ids[] = $agent_id;
		// 		// 	}
		// 		// }

		// 		// Fallback to old setting
		// 		if ( empty($agent_ids) ) {
		// 			$agent_ids =  get_option('bcoidx_agent_id');
		// 			$agent_ids = explode(',', $agent_ids);
		// 		}

		// 		$body['agentIds'] = $agent_ids;
		// 	}
		// }

		unset($body['agentOnly']);
		unset($body['officeOnly']);

		$this->body = $body;
	}

	// public function send_request_v1() {

	// 	$args = array(
	// 		'method' => 'POST',
	// 		'body' => json_encode($this->body),
	// 		'headers' => array(
	// 			'Authorization' => 'Bearer ' . base64_encode(Util::get_origin_domain()),
	// 			'Content-Type' => 'application/json'
	// 		),
	// 		'timeout' => 45,
	// 	);

	// 	$endpoint = PROPERTYPRESS_IDX_API . '/search-listings/';

	// 	$request = wp_remote_request($endpoint, $args);

	// 	if ( is_wp_error($request) || wp_remote_retrieve_response_code($request) !== 200 ) {
	// 		// error_log(print_r($request, true));
	// 		return $request;
	// 	}

	// 	$response = wp_remote_retrieve_body($request);

	// 	$output = array(
	// 		'args' => $args,
	// 		'endpoint' => $endpoint,
	// 		'request' => $request,
	// 		'response' => json_decode($response, true),
	// 	);

	// 	return $output;
	// }

	public function remove_duplicate_listings($response) {
		$option = get_option('bcoidx_show_duplicate_listings');

		if ( $option == 'show' ) {
			return $response;
		}

		if ( !empty($response['response']['response']['listings_response']['listings']) ) {

			$all_listings = $response['response']['response']['listings_response']['listings'];

			if ( $all_listings ) {

				$filtered = array_intersect_key($all_listings, array_unique(array_column($all_listings, 'streetAddress')));
				$temp = array();

				foreach ( $filtered as $listing ) {
					$temp[] = $listing;
				}

				$response['response']['response']['listings_response']['listings'] = $temp;
			}
		}

		return $response;
	}

	public function send_request_v2() {

		// if ( !defined('PROPERTYPRESS_IDX_API') ) {
		// 	define('PROPERTYPRESS_IDX_API', 'https://propertypress.com/wp-json/pp/v1');
		// }

		// $endpoint = PROPERTYPRESS_IDX_API . '/search-listings';

		$endpoint = 'https://api.brandco.com/api/wp-json/pp/v1/search-listings';

		if ( Util::isLocal() ) {
			$endpoint = 'https://apidash2.dev/api/wp-json/pp/v1/search-listings';
		}

		$ch = curl_init($endpoint);

		// curl_setopt($ch, CURLOPT_HTTPHEADER, array(
		// 	'Content-Type: application/json',
		// 	'Authorization: Bearer ' . base64_encode(Util::get_origin_domain()),
		// ));

		curl_setopt($ch, CURLOPT_HTTPHEADER, array(
			'Content-Type: application/json',
			'Authorization: Bearer 1234',
			'X-API-Key: ' . Util::get_origin_domain(),
		));

		$this->body['withCount'] = true;

		if ( empty($this->body['mlsIds']) ) {
			$this->body['mlsIds'] = Util::get_allowed_mls_ids();
		}

		if ( Util::isLocal() ) {
			curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
			curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		}

		curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($this->body));

		curl_setopt($ch, CURLOPT_TIMEOUT, 10);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

		$response = curl_exec($ch);
		curl_close($ch);

		$response = json_decode($response, true);

		// if ( !isset($response['response']['listings_response']['listings']) ) {
		// 	$error_body = json_decode($response['response']['body'], true);
		// 	$response['response']['error'] = $error_body['message'][0];
		// 	$response['response']['listings_response']['listings'] = array();
		// 	$response['response']['count_response']['count'] = 0;
		// }

		$output = array(
			'origin' => Util::get_origin_domain(),
			'endpoint' => $endpoint,
			'requested' => $this->body,
			'response' => $response,
			'original_body' => $this->original_body,
		);

		return $output;
	}

	public function sort($response) {

		if ( !empty($response['response']['response']['listings_response']['listings']) ) {
			$listings = $response['response']['response']['listings_response']['listings'];

			if ( !empty($this->original_body['queryOptions']['sort'][0]) ) {

				$sort = $this->original_body['queryOptions']['sort'][0];
				$field = $sort['field'];
				$direction = $sort['order'];

				if ( $field === 'price' && $direction === 'desc' ) {
					usort($listings, [$this, 'priceDesc']);
				}

				if ( $field === 'price' && $direction === 'asc' ) {
					usort($listings, [$this, 'priceAsc']);
				}

				if ( $field === 'listDate' && $direction === 'desc' ) {
					usort($listings, [$this, 'listedDesc']);
				}

				if ( $field === 'listDate' && $direction === 'asc' ) {
					usort($listings, [$this, 'listedAsc']);
				}

				$response['response']['response']['listings_response']['listings'] = $listings;
			}
		}

		return $response;
	}

	public function priceDesc($a, $b) {
		return $b['price'] - $a['price'];
	}

	public function priceAsc($a, $b) {
		return $a['price'] - $b['price'];
	}

	public function listedDesc($a, $b) {
		$dateA = strtotime($a['listDate']);
		$dateB = strtotime($b['listDate']);

		return $dateB - $dateA;
	}

	public function listedAsc($a, $b) {
		$dateA = strtotime($a['listDate']);
		$dateB = strtotime($b['listDate']);

		return $dateA - $dateB;
	}
}
