<?php
namespace BrandCo\Plugins\IDX;

class Shortcodes {

	public static function add_resources() {

		if ( defined('BRANDCO_IDX_PAGE') ) {
			return;
		}

		$js_registered = wp_script_is('brandco-idx-shortcodes-js');

		if ( !$js_registered ) {
			define('BRANDCO_IDX_PAGE', 'shortcode');
			Frontend::enqueue_js();
		}

		$primary_color = ( get_option('bcoidx_primary_color') ) ? get_option('bcoidx_primary_color') : '#D50000';
		$primary_color_dark = Util::adjustColor($primary_color, -50);
		$custom_css = "
			:root {
				--propertypressIDXprimaryColor: " . $primary_color . ";
				--propertypressIDXprimaryColorDark: " . $primary_color_dark . ";
			}
		";

		wp_enqueue_script('brandco-idx-shortcodes-js');
		wp_enqueue_style('brandco-idx-shortcodes-css');
		wp_add_inline_style('brandco-idx-shortcodes-css', $custom_css);
	}

	public static function alert_signup_form($atts) {
		self::add_resources();

		$shortcode = array(
			'template' => 'AlertSignup',
			'args' => array(
				'testing' => '1234',
			)
		);

		ob_start();
			?>
				<div bcoidx-shortcode-loader="<?php echo htmlspecialchars(json_encode($shortcode)); ?>">
					<div class="bcoidx-shortcode-loader" style="background: #eee; color: #999; text-align: center; padding: 20px; margin: 2px auto;">PropertyPress IDX Search Widget</div>
				</div>
			<?php
		$content = ob_get_contents();
		ob_end_clean();
		return $content;
	}

	public static function search_widget($atts) {

		self::add_resources();

		$shortcode = array(
			'template' => 'SearchWidget',
			'args' => array(
				'layout' => ( !empty($atts['layout']) ) ? $atts['layout'] : '',
				'min_price' => ( !empty($atts['min_price']) ) ? $atts['min_price'] : '',
				'max_price' => ( !empty($atts['max_price']) ) ? $atts['max_price'] : '',
				'property_types' => ( !empty($atts['property_types']) ) ? explode(',', $atts['property_types']) : '',
				'placeholder' => ( !empty($atts['placeholder']) ) ? htmlspecialchars($atts['placeholder']) : '',
				'mls' => ( !empty($atts['mls']) ) ? explode(',', $atts['mls']) : '',
			)
		);

		ob_start();
			?>
				<div bcoidx-shortcode-loader="<?php echo htmlspecialchars(json_encode($shortcode)); ?>"><div class="bcoidx-shortcode-loader" style="background: #eee; color: #999; text-align: center; padding: 20px; margin: 2px auto;">PropertyPress IDX Search Widget</div></div>
			<?php
		$content = ob_get_contents();
		ob_end_clean();
		return $content;
	}

	public static function listing_grid($atts) {

		self::add_resources();

		$args = $atts;

		$hotsheet_id = ( !empty($atts['savedsearch']) ) ? $atts['savedsearch'] : '';

		if ( $hotsheet_id ) {
			$atts['q'] = get_post_meta($hotsheet_id, 'bcoidx_hotsheet_search_parameters', true);
			$neighborhood_shortcode = get_post_meta($hotsheet_id, 'bcoidx_hotsheet_neighborhood', true);
			if ( $neighborhood_shortcode ) {
				$neighborhood_shortcode = explode(',', $neighborhood_shortcode);

				$neighborhoods[] = array(
					'neighborhood' => $neighborhood_shortcode[0],
					'city' => $neighborhood_shortcode[1],
					'state' => $neighborhood_shortcode[2],
				);

				$atts['q'] = base64_encode(json_encode(array('neighborhoodCityStates' => $neighborhoods)));
			}
		}

		$query = self::make_widget_query($atts);

		$args = array(
			'query' => base64_encode(json_encode($query)),
		);

		$include = false;
		if ( !empty($atts['include']) ) {
			$include = get_post_meta($atts['include'], 'bcoidx_hotsheet_search_parameters', true);
		}

		$include_manual_listings = false;
		$manual_listings = array();

		if ( !empty($atts['include_manual_listings']) ) {
			$manual_listings_categories = explode(',', trim($atts['include_manual_listings']));

			$manual_listings_query = get_posts(array(
				'post_type' => 'listing',
				'tax_query' => array(
					array(
						'taxonomy' => 'status',
						'terms' => $manual_listings_categories,
						'field' => 'name',
						'operator' => 'IN'
					)
				)
			));

			if ( $manual_listings_query ) {
				foreach ( $manual_listings_query as $wp_listing ) {
					$manual_listings[] = Util::wp_listing_to_idx_listing($wp_listing);
				}
			}
		}

		$args['show_days_on_site'] = false;
		if ( !empty($atts['show_days_on_site']) ) {
			$args['show_days_on_site'] = true;
		}

		$shortcode = array(
			'template' => 'ListingGrid',
			'args' => $args,
			'include' => $include,
			'manual_listings' => $manual_listings,
			'align' => ( !empty($atts['align']) ) ? $atts['align'] : 'none',
		);

		ob_start();
			?>
				<div bcoidx-shortcode-loader="<?php echo htmlspecialchars(json_encode($shortcode)); ?>"><div class="bcoidx-shortcode-loader" style="background: #eee; color: #999; text-align: center; padding: 20px; margin: 2px auto;">PropertyPress IDX Grid Widget</div></div>
			<?php
		$content = ob_get_contents();
		ob_end_clean();
		return $content;
	}

	public static function wnt_search() {

		self::add_resources();

		ob_start();
		echo do_shortcode('[idx_quick_search layout="simple"]');
		$content = ob_get_contents();
		ob_end_clean();
		return $content;
	}

	public static function wnt_featured() {

		self::add_resources();

		ob_start();
		echo do_shortcode('[idx_slider for="office"]');
		$content = ob_get_contents();
		ob_end_clean();
		return $content;
	}

	public static function wnt_grid($attr) {

		self::add_resources();

		$original_shortcode = json_encode($attr);
		// $area_found = false;
		// $area_names = array();
		$areas_not_found = array();
		$looked_for_area = false;

		$shortcode = '[idx_grid';

		$q = array();

		if ( !empty($attr['property_id']) ) {
			$shortcode .= ' mls_nums="' . $attr['property_id'] . '"';
			$shortcode .= ' statuses="1,2,3,4"';
		}

		if ( !empty($attr['min_bedrooms']) ) {
			$shortcode .= ' beds="' . $attr['min_bedrooms'] . '"';
		}

		if ( !empty($attr['min_price']) ) {
			$shortcode .= ' min_price="' . $attr['min_price'] . '"';
		}

		if ( !empty($attr['max_price']) ) {
			$shortcode .= ' max_price="' . $attr['max_price'] . '"';
		}

		if ( !empty($attr['maxresults']) ) {
			$shortcode .= ' limit="' . $attr['maxresults'] . '"';
		}

		if ( !empty($attr['ownertype']) ) {
		    if ( $attr['ownertype'] == 'broker' ) {
		        $shortcode .= ' for="office"';
		    }

		    if ( $attr['ownertype'] == 'agent_broker' ) {
		        $shortcode .= ' for="all"';
		    }

		    if ( $attr['ownertype'] == 'agent' ) {
		        $shortcode .= ' for="agent"';
		    }
		}

		$property_types = array();

		if ( !empty($attr['single_family']) ) {
			$property_types[] = 'Residential';
		}

		if ( !empty($attr['townhouse']) || !empty($attr['condo']) ) {
			$property_types[] = 'Condo/Townhome';
		}

		if ( !empty($attr['farm_hobby']) ) {
			$property_types[] = 'Farm/Ranch';
		}

		if ( !empty($attr['commercial']) ) {
			$property_types[] = 'Commercial Sale';
		}

		if ( !empty($attr['residential_lease']) ) {
			$property_types[] = 'Residential Lease';
		}

		if ( !empty($attr['lots_acreage']) ) {
			$property_types[] = 'Vacant Land';
		}

		if ( !empty($property_types) ) {
			$shortcode .= ' property_types="' . implode(',', $property_types) . '"';
		}

		if ( !empty($attr['built_before']) ) {
			$shortcode .= ' year_min="1500"';
			$shortcode .= ' year_max="' . $attr['built_before'] . '"';
		}

		if ( !empty($attr['new_construction']) ) {
			$last_year = date('Y') - 1;
			$shortcode .= ' year_min="' . $last_year . '"';
		}

		if ( !empty($attr['built_after']) ) {
			$shortcode .= ' year_min="' . $attr['built_after'] . '"';
		}

		if ( !empty($attr['zip_code']) ) {
			$shortcode .= ' postal_codes="' . $attr['zip_code'] . '"';
		}

		if ( !empty($attr['listing_status']) ) {
			if ( strpos($attr['listing_status'], 'sold') !== false ) {
				$shortcode .= ' statuses="4"';
			}
		}

		if ( !empty($attr['subdivision_text']) ) {
			$attr['subdivision'] = $attr['subdivision_text'];
		}

		if ( !empty($attr['subdivision']) ) {

			$looked_for_area = true;

			$subdivisions = explode('||', $attr['subdivision']);
			$neighborhoods = array();

			$subdiv = $subdivisions[0];

			$request = new AutocompleteRequest(array(
				'statuses' => 'active,pending,activeUnderContract',
				'fields' => 'districtCityState',
				'suggests' => 'districtCityState',
				'prefix' => $subdiv,
			));

			$response = $request->send_request_v2();
			$suggestions = $response['response']['response']['response'];

			if ( !$suggestions ) {

				$areas_not_found[] = $subdiv;

			} else {

				foreach ( $suggestions as $suggestion ) {

					if ( $suggestion['suggestType'] == 'districtCityState' ) {
						$fields = explode(', ', $suggestion['fieldValues']['districtCityState']);

						$neighborhoods[] = array(
							'neighborhood' => $fields[0],
							'city' => $fields[1],
							'state' => $fields[2],
						);
					}
				}

				$q['neighborhoodCityStates'] = $neighborhoods;
			}
		}

		if ( !empty($attr['county']) && empty($attr['city']) ) {

			$looked_for_area = true;

			$county = explode('||', $attr['county']);
			$county_array = array();

			$county = $county[0];

			$request = new AutocompleteRequest(array(
				'statuses' => 'active,pending,activeUnderContract',
				'fields' => 'countyState',
				'suggests' => 'countyState',
				'prefix' => $county,
			));

			$response = $request->send_request_v2();
			$suggestions = $response['response']['response']['response'];

			if ( ! $suggestions ) {

				$areas_not_found[] = $county;

			} else {

				foreach ( $suggestions as $suggestion ) {

					if ( $suggestion['suggestType'] == 'countyState' ) {
						$fields = explode(', ', $suggestion['text']);

						$county_array[] = array(
							'county' => $fields[0],
							'state' => $fields[1],
						);
					}
				}

				$q['countyStates'] = $county_array;
			}
		}

		if ( !empty($attr['school_district_name']) ) {

			$looked_for_area = true;

			$school_district_names = explode('||', $attr['school_district_name']);
			$schools = array();

			$school_district_name = $school_district_names[0];

			$request = new AutocompleteRequest(array(
				'statuses' => 'active,pending,activeUnderContract',
				'suggests' => 'schoolDistrict',
				'prefix' => $school_district_name,
			));

			$response = $request->send_request_v2();
			$suggestions = $response['response']['response']['response'];

			if ( !$suggestions ) {
				$areas_not_found[] = $school_district_name;
			} else {

				foreach ( $suggestions as $suggestion ) {
					if ( $suggestion['suggestType'] == 'schoolDistrict' ) {
						$schools[] = $suggestion['text'];
					}
				}

				$q['schoolDistricts'] = $schools;
			}
		}

		if ( !empty($attr['area_name']) ) {
			$attr['city'] = $attr['area_name'];
		}

		if ( !empty($attr['city']) ) {

			$looked_for_area = true;

			$request = new AutocompleteRequest(array(
				'statuses' => 'active,pending,activeUnderContract',
				'fields' => 'city,state',
				'suggests' => 'cityState',
				'prefix' => $attr['city'],
			));

			$response = $request->send_request_v2();

			$suggestions = $response['response']['response']['response'];

			if ( $suggestions ) {
				$cities = array();
				foreach ( $suggestions as $suggestion ) {

					if ( $suggestion['suggestType'] == 'cityState' ) {
						$cities[] = array(
							'city' => $suggestion['fieldValues']['city'],
							'state' => $suggestion['fieldValues']['state']
						);
					}
				}

				$q['cityStates'] = $cities;
			}

			if ( !$suggestions ) {
				$areas_not_found[] = $attr['city'];
			}
		}

		if ( !empty($attr['has_waterfront']) ) {
			$q['lotFeatures'] = array(
				'fields' => array(
					array(
						'excludeTerm' => false,
						'matchAny' => true,
						'term' => 'Waterfront'
					)
				),
			);
		}

		if ( !empty($attr['gated_community']) ) {
			$q['communityFeatures'] = array(
				'fields' => array(
					array(
						'excludeTerm' => false,
						'matchAny' => true,
						'term' => 'Gated'
					)
				),
			);
		}

		if ( !empty($attr['shortsale']) ) {
			$q['financialInformation'] = array(
				'fields' => array(
					array(
						'excludeTerm' => false,
						'matchAny' => true,
						'term' => 'Short Sale'
					)
				),
			);
		}

		if ( !empty($attr['foreclosure']) ) {
			$q['financialInformation'] = array(
				'fields' => array(
					array(
						'excludeTerm' => false,
						'matchAny' => true,
						'term' => 'Real Estate Owned',
					)
				),
			);
		}

		if ( !empty($q) ) {
			$q = base64_encode(json_encode($q));
			$shortcode .= ' q="' . $q . '"';
		}

		$shortcode .= ']';

		// Set to true if we aren't looking for an area at all
		// if ( empty($attr['subdivision']) && empty($attr['county']) && empty($attr['city']) && empty($attr['school_district_name']) ) {
		// 	$area_found = true;
		// }

		if ( $looked_for_area && $areas_not_found ) {
			$default_search = get_option('bcoidx_fallback_search');
			$none_found_text = get_option('bcoidx_none_found_text');
			ob_start();
			echo '<div style="display: none !important;">' . $original_shortcode . '</div>';
			echo '<p>' . $none_found_text . ' There are currently no listings in ' . implode(' or ', $areas_not_found) . ' but please check out these great listings:</p>';
			echo do_shortcode('[idx_slider q="' . $default_search . '" limit="10"]');
			$content = ob_get_contents();
			ob_end_clean();
			return $content;
		}

		ob_start();
		echo do_shortcode($shortcode);
		$content = ob_get_contents();
		ob_end_clean();
		return $content;
	}

	public static function listing_slider($atts) {

		self::add_resources();

		$args = $atts;

		$hotsheet_id = ( !empty($atts['savedsearch']) ) ? $atts['savedsearch'] : '' ;
		$include = false;

		if ( $hotsheet_id ) {
			$query = get_post_meta($hotsheet_id, 'bcoidx_hotsheet_search_parameters', true);
			$decoded_query = Util::decode_params($query);

			if ( !empty($atts['limit']) ) {
				$decoded_query['limit'] = (int) $atts['limit'];
			}

			$args = array(
				'query' => base64_encode(json_encode($decoded_query)),
			);
		} else {

			$query = self::make_widget_query($atts);

			if ( !empty($atts['include']) ) {
				$include = get_post_meta($atts['include'], 'bcoidx_hotsheet_search_parameters', true);
				$_decoded = Util::decode_params($include);
				$_decoded['limit'] = 10;

				if ( !empty($atts['limit']) ) {
					$_decoded['limit'] = (int) $atts['limit'];
				}

				$include = base64_encode(json_encode($_decoded));
			}

			$args = array(
				'query' => base64_encode(json_encode($query)),
			);
		}

		$args['show_days_on_site'] = false;
		if ( !empty($atts['show_days_on_site']) ) {
			$args['show_days_on_site'] = true;
		}

		$shortcode = array(
			'template' => 'ListingSlider',
			'args' => $args,
			'include' => $include,
		);

		ob_start();
			?>
				<div bcoidx-shortcode-loader="<?php echo htmlspecialchars(json_encode($shortcode)); ?>"><div class="bcoidx-shortcode-loader" style="background: #eee; color: #999; text-align: center; padding: 20px; margin: 2px auto;">PropertyPress IDX Slider Widget</div></div>
			<?php
		$content = ob_get_contents();
		ob_end_clean();
		return $content;
	}

	public static function sold_listings_grid($atts) {

		self::add_resources();

		$for = ( !empty($atts['for']) ) ? $atts['for'] : 'agent'; // agent, office

		$limit = 100;

		if ( !empty($atts['limit']) ) {
			$limit = (int) $atts['limit'];
		}

		$args = array(
			'statuses' => array(4),
			'limit' => $limit,
		);

		$args['queryOptions']['sort'] = array(
			array('field' => 'listDate',  'order' => 'desc')
		);

		if ( 'agent' == $for ) {
			$args['agentTypes'] = array('Listing Agent');
			$args['agentOnly'] = true;
		}

		if ( 'office' == $for ) {
			$args['officeOnly'] = true;
		}

		$shortcode = array(
			'template' => 'ListingGrid',
			'afterSort' => 'sold',
			'args' => array(
				'query' => base64_encode(json_encode($args))
			)
		);

		ob_start();
			?>
				<div bcoidx-shortcode-loader="<?php echo htmlspecialchars(json_encode($shortcode)); ?>"><div class="bcoidx-shortcode-loader" style="background: #eee; color: #999; text-align: center; padding: 20px; margin: 2px auto;">PropertyPress IDX Widget</div></div>
			<?php
		$content = ob_get_contents();
		ob_end_clean();
		return $content;
	}

	public static function featured_listings_grid($atts) {
		self::add_resources();

		$for = ( !empty($atts['for']) ) ? $atts['for'] : 'agent'; // agent, office

		$limit = 100;

		if ( !empty($atts['limit']) ) {
			$limit = (int) $atts['limit'];
		}

		$args = array(
			'statuses' => array(1, 2, 3),
			'limit' => $limit
		);

		if ( 'agent' == $for ) {
			$args['agentOnly'] = true;
		}

		if ( 'office' == $for ) {
			$args['officeOnly'] = true;
		}

		$shortcode = array(
			'template' => 'ListingGrid',
			'args' => array(
				'query' => base64_encode(json_encode($args))
			)
		);

		ob_start();
			?>
				<div bcoidx-shortcode-loader="<?php echo htmlspecialchars(json_encode($shortcode)); ?>"><div class="bcoidx-shortcode-loader" style="background: #eee; color: #999; text-align: center; padding: 20px; margin: 2px auto;">PropertyPress IDX Widget</div></div>
			<?php
		$content = ob_get_contents();
		ob_end_clean();
		return $content;
	}

	public static function make_widget_query($atts) {
		$query = array();

		if ( !empty($atts['q']) ) {
			$query = Util::decode_params($atts['q']);
		}

		$query['queryOptions']['sort'] = array(
			array('field' => 'listDate',  'order' => 'desc')
		);

		if ( !empty($atts['order']) ) {

			if ( $atts['order'] == 'low_to_high' ) {
				$query['queryOptions']['sort'] = array(
					array('field' => 'price',  'order' => 'asc')
				);
			}

			if ( $atts['order'] == 'high_to_low' ) {
				$query['queryOptions']['sort'] = array(
					array('field' => 'price',  'order' => 'desc')
				);
			}
		}

		if ( !empty($atts['mls']) ) {
			$query['mlsIds'] = explode(',', $atts['mls']);
		}

		if ( !empty($atts['year_min']) ) {
			$query['yearMin'] = $atts['year_min'];
		}

		if ( !empty($atts['year_max']) ) {
			$query['yearMax'] = $atts['year_max'];
		}

		if ( !empty($atts['property_types']) ) {
			$query['propertyTypes'] = explode(',', $atts['property_types']);
		}

		// if ( !empty($atts['property_types']) ) {
		// 	$query['propertyTypes'] = explode(',', $atts['property_types']);
		// }

		if ( !empty($atts['address']) ) {
			$query['streetAddress'] = $atts['address'];
		}

		if ( !empty($atts['statuses']) ) {
			$query['statuses'] = explode(',', $atts['statuses']);
		}

		if ( !empty($atts['openhouses']) ) {
			$query['requireOpenHouses'] = true;
		}

		if ( !empty($atts['limit']) ) {
			$query['limit'] = (int) $atts['limit'];
		}

		if ( !empty($atts['sqft_min']) ) {
			$query['sqFtMin'] = (int) $atts['sqft_min'];
		}

		if ( !empty($atts['sqft_max']) ) {
			$query['sqFtMax'] = (int) $atts['sqft_max'];
		}

		if ( !empty($atts['lot_size']) ) {
			$query['lotSizeMin'] = (float) $atts['lot_size'];
		}

		if ( !empty($atts['lot_size_max']) ) {
			$query['lotSizeMax'] = (float) $atts['lot_size_max'];
		}

		if ( !empty($atts['agent']) ) {
			$query['agentIds'] = explode(',', str_replace(' ', '', $atts['agent']));
			$query['agentTypes'] = array('Listing Agent');
		}

		if ( !empty($atts['office']) ) {
			$query['officeIds'] = explode(',', $atts['office']);
		}

		if ( !empty($atts['postal_codes']) ) {
			$query['postalCodes'] = explode(',', $atts['postal_codes']);
		}

		if ( !empty($atts['mls_nums']) ) {
			$query['mlsNums'] = explode(',', $atts['mls_nums']);
		}

		if ( !empty($atts['school_districts']) ) {
			$query['schoolDistricts'] = explode(',', $atts['school_districts']);
		}

		if ( !empty($atts['building_name']) ) {
			$query['buildingNames'] = explode(',', $atts['building_name']);
		}

		if ( !empty($atts['high_schools']) ) {
			$query['highSchools'] = explode(',', $atts['high_schools']);
		}

		if ( !empty($atts['middle_schools']) ) {
			$query['middleSchools'] = explode(',', $atts['middle_schools']);
		}

		if ( !empty($atts['for']) ) {

			if ( $atts['for'] == 'agent' ) {

				$query['agentOnly'] = true;

				if ( !empty($atts['agent_listed_only']) ) {
					$query['agentTypes'] = array('Listing Agent');
				}
			}

			if ( $atts['for'] == 'office' ) {
				// $query['officeIds'] = explode(',', get_option('bcoidx_office_id'));
				$query['officeOnly'] = true;
			}
		}

		if ( !empty($atts['area'])) {
			$request = new AutocompleteRequest(array(
				'statuses' => 'active,pending,activeUnderContract',
				'fields' => 'address,ln',
				'suggests' => 'districtCityState,postalCode,cityState,ln,elementarySchool,middleSchool,highSchool,mlsAreaMajor,fullAddress,schoolDistrict,countyState,buildingName',
				'prefix' => $atts['area'],
			));

			$response = $request->send_request_v2();

			$suggestions = $response['response']['response']['response'];

			if ( $suggestions ) {
				$suggestion = $suggestions[0];

				if ( 'fullAddress' == $suggestion['suggestType'] ) {
					// $query['streetAddress'] = $suggestion['text'];
					$query['mlsNums'] = array($suggestion['fieldValues']['ln']);
				}

				if ( 'postalCode' == $suggestion['suggestType'] ) {
					$query['postalCodes'] = array($suggestion['text']);
				}

				if ( 'ln' == $suggestion['suggestType'] ) {
					$query['mlsNums'] = array($suggestion['text']);
				}

				// if ( 'ln' == $suggestion['suggestType'] ) {
				// 	$query['mlsNums'] = array($suggestion['text']);
				// }

				if ( 'cityState' == $suggestion['suggestType'] ) {
					$parts = explode(', ', $suggestion['text']);
					$query['cityStates'] = array(
						array(
							'city' => $parts[0],
							'state' =>  $parts[1],
						)
					);
				}

				if ( 'districtCityState' == $suggestion['suggestType'] ) {
					$parts = explode(', ', $suggestion['text']);
					$query['neighborhoodCityStates'] = array(
						array(
							'neighborhood' => $parts[0],
							'city' => $parts[1],
							'state' =>  $parts[2],
						)
					);
				}

				if ( 'countyState' == $suggestion['suggestType'] ) {
					$parts = explode(', ', $suggestion['text']);
					$query['countyState'] = array(
						array(
							'county' => $parts[0],
							'state' =>  $parts[1],
						)
					);
				}
			}
		}

		if ( !empty($atts['min_price']) ) {
			$query['priceMin'] = $atts['min_price'];
		}

		if ( !empty($atts['max_price']) ) {
			$query['priceMax'] = $atts['max_price'];
		}

		if ( !empty($atts['beds']) ) {
			$query['bedsMin'] = $atts['beds'];
		}

		if ( !empty($atts['baths']) ) {
			$query['bathsMin'] = $atts['baths'];
		}

		if ( !empty($atts['view']) ) {
			$query['view'] = array(
				'fields' => array(
					array(
						'excludeTerm' => false,
						'matchAny' => true,
						'term' => $atts['view'],
					)
				),
			);
		}

		if ( !empty($atts['property_sub_type']) ) {
			$query['propertySubTypes'] = array(
				'fields' => array(
					array(
						'excludeTerm' => false,
						'matchAny' => true,
						'term' => $atts['property_sub_type']
					)
				),
			);
		}

		if ( !empty($atts['waterfront']) ) {
			$query['lotFeatures'] = array(
				'fields' => array(
					array(
						'excludeTerm' => false,
						'matchAny' => true,
						'term' => 'Waterfront'
					)
				),
			);
		}

		if ( !empty($atts['new_construction']) ) {
			$query['legacyFields'] = array(
				'fields' => array(
					array(
						'excludeTerm' => false,
						'matchAny' => true,
						'term' => 'isNewConstruction'
					)
				),
			);
		}

		if ( !empty($atts['foreclosures']) ) {
			$query['legacyFields'] = array(
				'fields' => array(
					array(
						'excludeTerm' => false,
						'matchAny' => true,
						'term' => 'isForeclosure'
					)
				),
			);
		}

		if ( !empty($atts['county']) ) {

			$request = new AutocompleteRequest(array(
				'statuses' => 'active,pending,activeUnderContract',
				'fields' => 'countyState',
				'suggests' => 'countyState',
				'prefix' => $atts['county'],
			));

			$response = $request->send_request_v2();
			$suggestions = $response['response']['response']['response'];

			if ( $suggestions ) {
				foreach ( $suggestions as $suggestion ) {

					if ( $suggestion['suggestType'] == 'countyState' ) {
						$fields = explode(', ', $suggestion['text']);

						$county_array[] = array(
							'county' => $fields[0],
							'state' => $fields[1],
						);
					}
				}

				$query['countyStates'] = $county_array;
			}
		}

		if ( !empty($atts['city']) ) {

			$request = new AutocompleteRequest(array(
				'statuses' => 'active,pending,activeUnderContract',
				'fields' => 'cityState',
				'suggests' => 'cityState',
				'prefix' => $atts['city'],
			));

			$response = $request->send_request_v2();
			$suggestions = $response['response']['response']['response'];

			if ( $suggestions ) {
				foreach ( $suggestions as $suggestion ) {

					if ( $suggestion['suggestType'] == 'cityState' ) {
						$fields = explode(', ', $suggestion['text']);

						$array[] = array(
							'city' => $fields[0],
							'state' => $fields[1],
						);
					}
				}

				$query['cityState'] = $array;
			}
		}

		return $query;
	}
}
