<?php
namespace BrandCo\Plugins\IDX;

class Util {

	public static function mls_agent_list($mls_metadata) {
		$mls_agent_list = array();

		if ( !is_array($mls_metadata) ) {
			return $mls_agent_list;
		}

		foreach ( $mls_metadata as $_mls ) {

			if ( !is_array($_mls) ) {
				continue;
			}

			$_mls = reset($_mls);
			$agents_ids = get_option('bcoidx_agent_id_' . $_mls['mlsId']);

			if ( $agents_ids ) {
				$agents_ids = explode(',', $agents_ids);

				$mls = array(
					'mls' => $_mls['mlsId'],
					'agents' => $agents_ids,
				);

				$mls_agent_list[] = $mls;
			}
		}

		return $mls_agent_list;
	}

	public static function mls_office_list($mls_metadata) {
		$mls_office_list = array();

		if ( !is_array($mls_metadata) ) {
			return $mls_office_list;
		}

		foreach ( $mls_metadata as $_mls ) {

			if ( !is_array($_mls) ) {
				continue;
			}

			$_mls = reset($_mls);
			$offices_ids = get_option('bcoidx_office_id_' . $_mls['mlsId']);

			if ( $offices_ids ) {
				$offices_ids = explode(',', $offices_ids);

				$mls = array(
					'mls' => $_mls['mlsId'],
					'offices' => $offices_ids,
				);

				$mls_office_list[] = $mls;
			}
		}

		return $mls_office_list;
	}

	public static function get_listing_id_from_detail_page_slug($slug) {
		$slug_parts = explode('--', $slug);
		return $slug_parts[count($slug_parts) - 1];
	}

	public static function make_listing_slug($listing) {
		if ( empty($listing['streetAddress']) ) {
			return;
		}

		$slug = $listing['streetAddress'] . '-' . $listing['city'] . '-' . $listing['state'];
		$slug = sanitize_title($slug);
		$slug .= '--' . $listing['mlsNum'];
		return $slug;
	}

	public static function decode_polygon_data_from_search_parameters($search_parameters) {

		if ( !empty($search_parameters['polygon']) ) {
			$search_parameters['polygon'] = json_decode(base64_decode($search_parameters['polygon']), true);
		}

		return $search_parameters;
	}

	public static function app_url($path = '/') {
		return get_option('siteurl') . '/' . get_option('bcoidx_app_url') . $path;
	}

	public static function get_template($path, $args) {
		include BRANDCO_IDX_PLUGIN_PATH . 'public/views/' . $path . '.php';
	}

	public static function render_template($path, $args) {
		ob_start();
		include BRANDCO_IDX_PLUGIN_PATH . 'public/views/' . $path . '.php';
		$content = ob_get_contents();
		ob_end_clean();
		return $content;
	}

	public static function get_mls_array() {
		return explode(',', trim(get_option('bcoidx_mls')));
	}

	public static function decode_params($encoded_params) {
		$parameters = base64_decode($encoded_params);
		$parameters = json_decode(html_entity_decode(stripslashes($parameters)), true);

		return $parameters;
	}

	public static function get_origin_domain() {
		global $remote_origin;

		if ( $remote_origin ) {
			return $remote_origin;
		}

		if ( get_option('bcoidx_app_test_url') ) {
			return get_option('bcoidx_app_test_url');
		}

		return apply_filters('bcoidx_origin_domain', $_SERVER['HTTP_HOST']);
	}

	public static function request_args($type) {
		return array(
			'method' => $type,
			'headers' => array(
				'Authorization' => 'Bearer ' . base64_encode(Util::get_origin_domain()),
				'Content-Type' => 'application/json'
			),
			'timeout' => 15,
		);
	}

	public static function adjustColor($hex, $steps) {
		// Steps should be between -255 and 255. Negative = darker, positive = lighter
		$steps = max(-255, min(255, $steps));

		// Normalize into a six character long hex string
		$hex = str_replace('#', '', $hex);
		if (strlen($hex) == 3) {
		    $hex = str_repeat(substr($hex,0,1), 2).str_repeat(substr($hex,1,1), 2).str_repeat(substr($hex,2,1), 2);
		}

		// Split into three parts: R, G and B
		$color_parts = str_split($hex, 2);
		$return = '#';

		foreach ($color_parts as $color) {
		    $color   = hexdec($color); // Convert to decimal
		    $color   = max(0,min(255,$color + $steps)); // Adjust color
		    $return .= str_pad(dechex($color), 2, '0', STR_PAD_LEFT); // Make two char hex code
		}

		return $return;
	}

	public static function get_my_alerts($user_id) {

		$savedposts = get_posts(array(
			'post_type' => array(
				'bcoidx_savedsearch',
				'bcoidx_favlistings'
			),
			'post_status' => 'publish',
			'posts_per_page' => -1,
			'post_author' => $user_id,
		));

		$array = array();

		if ( $savedposts ) {
			foreach ( $savedposts as $savedpost ) {
				$type = $savedpost->post_type;

				if ( $type == 'bcoidx_savedsearch' ) {
					$br_id = get_post_meta($savedpost->ID, 'bcoidx_saved_search_brivityalertid', true);
				}

				if ( $type == 'bcoidx_favlistings' ) {
					$br_id = get_post_meta($savedpost->ID, 'bcoidx_favorite_listing_brivityalertid', true);
				}

				$array[] = array(
					'type' => $type,
					'id' => $savedpost->ID,
					'br_id' => $br_id
				);
			}
		}

		return $array;
	}

	public static function get_alert_by_alert_id($alert_id) {

		$query = get_posts(array(
			'post_type' => array(
				'bcoidx_savedsearch',
				'bcoidx_favlistings'
			),
			'post_status' => 'publish',
			'posts_per_page' => 10,
			'fields' => 'ids',
			'meta_query' => array(
				'relation' => 'OR',
				array(
					'key' => 'bcoidx_saved_search_brivityalertid',
					'value' => $alert_id,
				),
				array(
					'key' => 'bcoidx_favorite_listing_brivityalertid',
					'value' => $alert_id,
				)
			)
		));

		if ( $query ) {
			$query = $query[0];
		} else {
			$query = false;
		}

		return $query;
	}

	public static function wp_listing_to_idx_listing($wp_post) {
		$thumbnail = get_the_post_thumbnail_url($wp_post, 'full');
		$property_type = wp_get_post_terms($wp_post->ID, 'property-types');
		if ( !empty($property_type) ) {
			$property_type = $property_type[0]->name;
		}

		$status = wp_get_post_terms($wp_post->ID, 'status');
		if ( !empty($status) ) {
			$status = $status[0]->name;
		}

		$streetAddress = get_post_meta($wp_post->ID, '_listing_address', true);
		$city = get_post_meta($wp_post->ID, '_listing_city', true);
		$state = get_post_meta($wp_post->ID, '_listing_state', true);
		$postalCode = get_post_meta($wp_post->ID, '_listing_zip', true);
		$openhouse = get_post_meta($wp_post->ID, '_listing_open_house', true);
		$mlsnumber = get_post_meta($wp_post->ID, '_listing_mls', true);
		$price = get_post_meta($wp_post->ID, '_listing_price', true);
		$price = str_replace('$', '', $price);
		$price = str_replace(',', '', $price);
		$bedrooms = get_post_meta($wp_post->ID, '_listing_bedrooms', true);
		$bathsTotalDecimal = get_post_meta($wp_post->ID, '_listing_bathrooms', true);
		$squareFeet = get_post_meta($wp_post->ID, '_listing_sqft', true);

		$listing = array();
		$listing['streetAddress'] = $streetAddress;
		$listing['city'] = $city;
		$listing['state'] = $state;
		$listing['postalCode'] = $postalCode;
		$listing['openhouses'] = ( $openhouse ) ? array($openhouse) : false;
		$listing['listDate'] = $wp_post->post_date;
		$listing['blueroofMlsId'] = ( $mlsnumber ) ? $mlsnumber : 'Custom';
		$listing['price'] = (float) $price;
		$listing['soldPrice'] = (float) $price;
		$listing['bedrooms'] = $bedrooms;
		$listing['bathsTotalDecimal'] = $bathsTotalDecimal;
		$listing['squareFeet'] = $squareFeet;
		$listing['listOfficeName'] = '';
		$listing['standardStatus'] = $status;
		$listing['standardPropertyType'] = $property_type;
		$listing['custom_photo'] = $thumbnail;
		$listing['photosAvailable'] = ($thumbnail) ? 1 : 0;
		$listing['custom_url'] = get_permalink($wp_post);
		$listing['photos'] = array(array('url' => $thumbnail));

		if ( defined('WPSEO_VERSION') ) {

			$primary_status = get_post_meta($wp_post->ID, '_yoast_wpseo_primary_status', true);
			if ( $primary_status ) {
				$primary_status = get_term_by('term_id', $primary_status, 'status');
				$listing['standardStatus'] = $primary_status->name;
			}

			$primary_type = get_post_meta($wp_post->ID, '_yoast_wpseo_primary_property-types', true);
			if ( $primary_type ) {
				$primary_type = get_term_by('term_id', $primary_type, 'property-types');
				$listing['standardPropertyType'] = $primary_type->name;
			}
		}

		return $listing;
	}

	public static function get_allowed_mls_ids() {
		$mlsData = MLSMetadata::get();

		if ( !empty($mlsData[0]) ) {
			$mlsIds = array_map(function($mls) {
				$keys = array_keys($mls);
				return ( !empty($keys) ) ? $keys[0] : false;
			}, $mlsData);

			return $mlsIds;
		}

		return [];
	}

	public static function isLocal() {
		return str_contains($_SERVER['SERVER_NAME'], '.local');
	}
}
